#!/usr/bin/python
# -*- coding=utf-8 -*-
#-------------------------------------------------------------------------------
# Name:        stem_verb_const
# Purpose:     Arabic lexical analyser constants, provides feature for stemming arabic word as verb
#
# Author:      Taha Zerrouki (taha.zerrouki[at]gmail.com)
#
# Created:     31-10-2011
# Copyright:   (c) Taha Zerrouki 2011
# Licence:     GPL
#-------------------------------------------------------------------------------
"""
Arabic lexical analyser constants, provides feature for stemming arabic word as verb
"""
import re
import pyarabic.araby as ar
import libqutrub.verb_const as qutrubVerbConst
VERB_STAMP_PAT = re.compile(u"[%s%s%s%s%s%s]"%(ar.ALEF, ar.YEH, ar.WAW, ar.ALEF_MAKSURA,
                         ar.HAMZA, ar.SHADDA), re.UNICODE)
# Compound affixes
COMP_PREFIX_LETTERS = u"أسلفو"
COMP_SUFFIX_LETTERS = u"ينهكماو"
COMP_INFIX_LETTERS = u""
COMP_MAX_PREFIX = 3
COMP_MAX_SUFFIX = 6
COMP_MIN_STEM = 2
COMP_JOKER = u"*"
"""COMP_PREFIX_LIST=set([
    "",
    ar.ALEF_HAMZA_ABOVE,
    ar.ALEF_HAMZA_ABOVE+ ar.FEH,
    ar.ALEF_HAMZA_ABOVE+ ar.FEH+ ar.LAM,
    ar.ALEF_HAMZA_ABOVE+ ar.FEH+ ar.SEEN,
    ar.ALEF_HAMZA_ABOVE+ ar.WAW,
    ar.ALEF_HAMZA_ABOVE+ ar.WAW+ ar.SEEN,
    ar.ALEF_HAMZA_ABOVE+ ar.WAW+ ar.LAM,
    ar.ALEF_HAMZA_ABOVE+ ar.LAM,
    ar.ALEF_HAMZA_ABOVE+ ar.SEEN,
    ar.FEH,
    ar.FEH+ ar.LAM,
    ar.FEH+ ar.SEEN,
    ar.WAW,
    ar.WAW+ ar.LAM,
    ar.WAW+ ar.SEEN,
    ar.LAM,
    ar.SEEN,
    ]);
COMP_SUFFIX_LIST=set(["",
    ar.YEH,
    ar.NOON+ ar.YEH,
    ar.NOON+ ar.ALEF,
    ar.KAF,
    ar.KAF+ ar.MEEM+ ar.ALEF,
    ar.KAF+ ar.MEEM,
    ar.KAF+ ar.NOON,
    ar.HEH,
    ar.HEH+ ar.ALEF,
    ar.HEH+ ar.MEEM+ ar.ALEF,
    ar.HEH+ ar.MEEM,
    ar.HEH+ ar.NOON,
# To double MAf3ool suffix
    u'يي',
    u'يني',
    u'ينا',
    u'يك',
    u'يكما',
    u'يكم',
    u'يكن',
    u'يه',
    u'يها',
    u'يهما',
    u'يهم',
    u'يهن',
    u'نيي',
    u'نيني',
    u'نينا',
    u'نيك',
    u'نيكما',
    u'نيكم',
    u'نيكن',
    u'نيه',
    u'نيها',
    u'نيهما',
    u'نيهم',
    u'نيهن',
    u'ناي',
    u'ناني',
    u'نانا',
    u'ناك',
    u'ناكما',
    u'ناكم',
    u'ناكن',
    u'ناه',
    u'ناها',
    u'ناهما',
    u'ناهم',
    u'ناهن',
    u'كي',
    u'كني',
    u'كنا',
    u'كك',
    u'ككما',
    u'ككم',
    u'ككن',
    u'كه',
    u'كها',
    u'كهما',
    u'كهم',
    u'كهن',
    u'كماي',
    u'كماني',
    u'كمانا',
    u'كماك',
    u'كماكما',
    u'كماكم',
    u'كماكن',
    u'كماه',
    u'كماها',
    u'كماهما',
    u'كماهم',
    u'كماهن',
    u'كموي',
    u'كموني',
    u'كمونا',
    u'كموك',
    u'كموكما',
    u'كموكم',
    u'كموكن',
    u'كموه',
    u'كموها',
    u'كموهما',
    u'كموهم',
    u'كموهن',
    u'كني',
    u'كنني',
    u'كننا',
    u'كنك',
    u'كنكما',
    u'كنكم',
    u'كنكن',
    u'كنه',
    u'كنها',
    u'كنهما',
    u'كنهم',
    u'كنهن',
    u'هي',
    u'هني',
    u'هنا',
    u'هك',
    u'هكما',
    u'هكم',
    u'هكن',
    u'هه',
    u'هها',
    u'ههما',
    u'ههم',
    u'ههن',
    u'هاي',
    u'هاني',
    u'هانا',
    u'هاك',
    u'هاكما',
    u'هاكم',
    u'هاكن',
    u'هاه',
    u'هاها',
    u'هاهما',
    u'هاهم',
    u'هاهن',
    u'هماي',
    u'هماني',
    u'همانا',
    u'هماك',
    u'هماكما',
    u'هماكم',
    u'هماكن',
    u'هماه',
    u'هماها',
    u'هماهما',
    u'هماهم',
    u'هماهن',
    u'هموي',
    u'هموني',
    u'همونا',
    u'هموك',
    u'هموكما',
    u'هموكم',
    u'هموكن',
    u'هموه',
    u'هموها',
    u'هموهما',
    u'هموهم',
    u'هموهن',
    u'هني',
    u'هنني',
    u'هننا',
    u'هنك',
    u'هنكما',
    u'هنكم',
    u'هنكن',
    u'هنه',
    u'هنها',
    u'هنهما',
    u'هنهم',
    u'هنهن',
    ]);
"""
COMP_PREFIX_LIST_TAGS = {
    u"": {
        'tags': (u"", ),
        "vocalized": (u"", )
    },
    u'أ': {
        'tags': (u'استفهام', ),
        "vocalized": (u"أَ", )
    },
    u'أس': {
        'tags': (u'استفهام', u'استقبال'),
        "vocalized": (u"أَسَ", )
    },
    u'و': {
        'tags': (u"عطف", ),
        "vocalized": (u"وَ", )
    },
    u'وس': {
        'tags': (u"عطف", u'استقبال'),
        "vocalized": (u"وَسَ", )
    },
    u'أو': {
        'tags': (u'استفهام', ),
        "vocalized": (u"أَوََ", )
    },
    u'أوس': {
        'tags': (u'استفهام', u'استقبال'),
        "vocalized": (u"أَوََسَ", )
    },
    u'أول': {
        'tags': (u'استفهام', u'عطف', u'لام'),
        "vocalized": (u"أَوَلََ", )
    },
    u'س': {
        'tags': (u'استقبال', ),
        "vocalized": (u"سَ", )
    },
    u'ف': {
        'tags': (u'عطف', ),
        "vocalized": (u"فَ", )
    },
    u'فس': {
        'tags': (u'عطف', u'استقبال'),
        "vocalized": (u"فَسَ", )
    },
    u'أف': {
        'tags': (
            u'استفهام',
            u'عطف',
        ),
        "vocalized": (u"أَفَ", )
    },
    u'أفس': {
        'tags': (
            u'استفهام',
            u'عطف',
            u'استقبال',
        ),
        "vocalized": (u"أَفَسَ", )
    },
    u'ل': {
        'tags': (u'لام', ),
        "vocalized": (u"لِ", u"لَ" )
    },
    u'ول': {
        'tags': (u'عطف', u'لام'),
        "vocalized": (u"وَلََ", )
    },
    u'فل': {
        'tags': (u'عطف', u'لام'),
        "vocalized": (u"فَلَ", u"فَلِ", u'فَلْ')
    },
}
COMP_PREFIX_LIST = list(COMP_PREFIX_LIST_TAGS.keys())
COMP_SUFFIX_LIST_TAGS = {
    "": {
        'tags': (u"", ),
        "vocalized": (u"", )
    },
    u'ني': {
        'tags': (u"مفعول به", ),
        "vocalized": (u"نِي", ),
    },
    u'ي': {
        'tags': (u"مفعول به", ),
        "vocalized": (u"ي", ),
    },
    u'ك': {
        'tags': (u"مفعول به", ),
        "vocalized": (u"كَ", u"كِ", ),
    },
    u'ه': {
        'tags': (u"مفعول به", ),
        "vocalized": (u"هُ", ),
    },
    u'كم': {
        'tags': (u"مفعول به", ),
        "vocalized": (u"كُمْ", u"كُمُ"),
    },
    u'كن': {
        'tags': (u"مفعول به", ),
        "vocalized": (u"كُنَّ", ),
    },
    u'ها': {
        'tags': (u"مفعول به", ),
        "vocalized": (u"هَا", ),
    },
    u'هم': {
        'tags': (u"مفعول به", ),
        "vocalized": (u"هُمْ", u"هُمُ" ),
    },
    u'هن': {
        'tags': (u"مفعول به", ),
        "vocalized": (u"هُنَّ", ),
    },
    u'نا': {
        'tags': (u"مفعول به", ),
        "vocalized": (u"نَا", ),
    },
    u'كما': {
        'tags': (u"مفعول به", ),
        "vocalized": (u"كُمَا", ),
    },
    u'هما': {
        'tags': (u"مفعول به", ),
        "vocalized": (u"هُمَا", ),
    },
}
COMP_SUFFIX_LIST = list(COMP_SUFFIX_LIST_TAGS.keys())

CONJ_PREFIX_LETTERS = u"تأنياء"
CONJ_SUFFIX_LETTERS = u"متواني"
CONJ_INFIX_LETTERS = u""
CONJ_MAX_PREFIX = 1
CONJ_MAX_SUFFIX = 3
CONJ_MIN_STEM = 2
CONJ_JOKER = u"*"
CONJ_PREFIX_LIST = ("", ar.ALEF, ar.YEH, ar.TEH, ar.NOON,
                    ar.ALEF_HAMZA_ABOVE, ar.HAMZA)
CONJ_SUFFIX_LIST = (
    "",
    ar.TEH,
    ar.TEH + ar.ALEF,
    ar.TEH + ar.MEEM,
    ar.TEH + ar.MEEM + ar.WAW,
    ar.TEH + ar.MEEM + ar.ALEF,
    ar.TEH + ar.NOON,
    ar.ALEF,
    ar.NOON,
    ar.NOON + ar.ALEF,
    ar.ALEF + ar.NOON,
    ar.WAW + ar.ALEF,
    ar.WAW + ar.NOON,
    ar.WAW,
    ar.YEH,
    ar.YEH + ar.NOON,
)
SUFFIX_LIST_STRIPOUS = set([
    u"",
    ar.TEH,
    ar.TEH + ar.ALEF,
    ar.WAW + ar.ALEF,
    ar.WAW + ar.NOON,
    ar.WAW,
    ar.YEH,
    ar.YEH + ar.NOON,
])
SUFFIX_LIST_SAKEN = set([
    u"",
    ar.TEH,
    # حالة خاصة مع الفعل الناقص
    ar.TEH + ar.ALEF,
    ar.TEH + ar.MEEM,
    #TEH+MEEM+WAW,
    ar.TEH + ar.MEEM + ar.ALEF,
    ar.TEH + ar.NOON,
    ar.NOON,
    ar.NOON + ar.ALEF,
])
SUFFIX_LIST_VOWELED = set([
    u"",
    ar.ALEF,
    ar.ALEF + ar.NOON,
    ar.NOON,
    ar.TEH,
    ar.TEH + ar.NOON,
    ar.TEH + ar.MEEM,
])
PREFIX_LIST_STRIPOUS = (ar.YEH, ar.TEH, ar.NOON,
                        ar.ALEF_HAMZA_ABOVE, ar.HAMZA)
VERBAL_CONJUGATION_AFFIX = set([
    u'-',
    u'-ا',
    u'-ت',
    u'-تا',
    u'-تم',
    u'-تما',
    u'-تنّ',
    u'-تن',
    u'-ن',
    u'-نا',
    u'-وا',
    u'أ-',
    u'أ-نّ',
    u'ا-',
    u'ا-ا',
    u'ا-انّ',
    u'ا-ن',
    u'ا-نانّ',
    u'ا-نّ',
    u'ا-وا',
    u'ا-ي',
    u'ت-',
    u'ت-ا',
    u'ت-ان',
    u'ت-انّ',
    u'ت-ن',
    u'ت-نانّ',
    u'ت-نّ',
    u'ت-وا',
    u'ت-ون',
    u'ت-ي',
    u'ت-ين',
    u'ن-',
    u'ن-نّ',
    u'ي-',
    u'ي-ا',
    u'ي-ان',
    u'ي-انّ',
    u'ي-ن',
    u'ي-نانّ',
    u'ي-نّ',
    u'ي-وا',
    u'ي-ون',
    # تمو
    ##u"-تمو",
    ##u'ي-و',
    ##u'ت-و',
    ##u'ا-و',
    ##u'-و',
    u'-ي',
    # added confirmed
    u'-تن',
    u'أ-ن',
    u'ا-ان',
    u'ا-نان',
    u'ا-ن',
    u'ت-ان',
    u'ت-نان',
    u'ت-ن',
    u'ن-ن',
    u'ي-ان',
    u'ي-نان',
    u'ي-ن',
    u'ء-ن',
    u'ء-',
])
TABLE_DOUBLE_TRANSITIVE_SUFFIX = {
    u'يك': {
        'first': u'ي',
        'second': u'ي'
    },
    u'يكما': {
        'first': u'ي',
        'second': u'ي'
    },
    u'يكم': {
        'first': u'ي',
        'second': u'ي'
    },
    u'يكن': {
        'first': u'ي',
        'second': u'ي'
    },
    u'يه': {
        'first': u'ي',
        'second': u'ي'
    },
    u'يها': {
        'first': u'ي',
        'second': u'ي'
    },
    u'يهما': {
        'first': u'ي',
        'second': u'ي'
    },
    u'يهم': {
        'first': u'ي',
        'second': u'ي'
    },
    u'يهن': {
        'first': u'ي',
        'second': u'ي'
    },
    u'نيي': {
        'first': u'ني',
        'second': u'ني'
    },
    u'نيني': {
        'first': u'ني',
        'second': u'ني'
    },
    u'نينا': {
        'first': u'ني',
        'second': u'ني'
    },
    u'نيك': {
        'first': u'ني',
        'second': u'ني'
    },
    u'نيكما': {
        'first': u'ني',
        'second': u'ني'
    },
    u'نيكم': {
        'first': u'ني',
        'second': u'ني'
    },
    u'نيكن': {
        'first': u'ني',
        'second': u'ني'
    },
    u'نيه': {
        'first': u'ني',
        'second': u'ني'
    },
    u'نيها': {
        'first': u'ني',
        'second': u'ني'
    },
    u'نيهما': {
        'first': u'ني',
        'second': u'ني'
    },
    u'نيهم': {
        'first': u'ني',
        'second': u'ني'
    },
    u'نيهن': {
        'first': u'ني',
        'second': u'ني'
    },
    u'ناي': {
        'first': u'نا',
        'second': u'نا'
    },
    u'ناني': {
        'first': u'نا',
        'second': u'نا'
    },
    u'نانا': {
        'first': u'نا',
        'second': u'نا'
    },
    u'ناك': {
        'first': u'نا',
        'second': u'نا'
    },
    u'ناكما': {
        'first': u'نا',
        'second': u'نا'
    },
    u'ناكم': {
        'first': u'نا',
        'second': u'نا'
    },
    u'ناكن': {
        'first': u'نا',
        'second': u'نا'
    },
    u'ناه': {
        'first': u'نا',
        'second': u'نا'
    },
    u'ناها': {
        'first': u'نا',
        'second': u'نا'
    },
    u'ناهما': {
        'first': u'نا',
        'second': u'نا'
    },
    u'ناهم': {
        'first': u'نا',
        'second': u'نا'
    },
    u'ناهن': {
        'first': u'نا',
        'second': u'نا'
    },
    u'كي': {
        'first': u'ك',
        'second': u'ك'
    },
    u'كنا': {
        'first': u'ك',
        'second': u'ك'
    },
    ##u'كك':{'first':u'ك','second':u'ك'},
    ##u'ككما':{'first':u'ك','second':u'ك'},
    ##u'ككم':{'first':u'ك','second':u'ك'},
    ##u'ككن':{'first':u'ك','second':u'ك'},
    u'كه': {
        'first': u'ك',
        'second': u'ك'
    },
    u'كها': {
        'first': u'ك',
        'second': u'ك'
    },
    u'كهما': {
        'first': u'ك',
        'second': u'ك'
    },
    u'كهم': {
        'first': u'ك',
        'second': u'ك'
    },
    u'كهن': {
        'first': u'ك',
        'second': u'ك'
    },
    u'كماي': {
        'first': u'كما',
        'second': u'كما'
    },
    u'كماني': {
        'first': u'كما',
        'second': u'كما'
    },
    u'كمانا': {
        'first': u'كما',
        'second': u'كما'
    },
    ##u'كماك':{'first':u'كما','second':u'كما'},
    ##u'كماكما':{'first':u'كما','second':u'كما'},
    ##u'كماكم':{'first':u'كما','second':u'كما'},
    ##u'كماكن':{'first':u'كما','second':u'كما'},
    u'كماه': {
        'first': u'كما',
        'second': u'كما'
    },
    u'كماها': {
        'first': u'كما',
        'second': u'كما'
    },
    u'كماهما': {
        'first': u'كما',
        'second': u'كما'
    },
    u'كماهم': {
        'first': u'كما',
        'second': u'كما'
    },
    u'كماهن': {
        'first': u'كما',
        'second': u'كما'
    },
    u'كموي': {
        'first': u'كم',
        'second': u'كم'
    },
    u'كموني': {
        'first': u'كم',
        'second': u'كم'
    },
    u'كمونا': {
        'first': u'كم',
        'second': u'كم'
    },
    u'كموك': {
        'first': u'كم',
        'second': u'كم'
    },
    u'كموكما': {
        'first': u'كم',
        'second': u'كم'
    },
    u'كموكم': {
        'first': u'كم',
        'second': u'كم'
    },
    u'كموكن': {
        'first': u'كم',
        'second': u'كم'
    },
    u'كموه': {
        'first': u'كم',
        'second': u'كم'
    },
    u'كموها': {
        'first': u'كم',
        'second': u'كم'
    },
    u'كموهما': {
        'first': u'كم',
        'second': u'كم'
    },
    u'كموهم': {
        'first': u'كم',
        'second': u'كم'
    },
    u'كموهن': {
        'first': u'كم',
        'second': u'كم'
    },
    u'كني': {
        'first': u'كن',
        'second': u'كن'
    },
    u'كنني': {
        'first': u'كن',
        'second': u'كن'
    },
    u'كننا': {
        'first': u'كن',
        'second': u'كن'
    },
    u'كنك': {
        'first': u'كن',
        'second': u'كن'
    },
    u'كنكما': {
        'first': u'كن',
        'second': u'كن'
    },
    u'كنكم': {
        'first': u'كن',
        'second': u'كن'
    },
    u'كنكن': {
        'first': u'كن',
        'second': u'كن'
    },
    u'كنه': {
        'first': u'كن',
        'second': u'كن'
    },
    u'كنها': {
        'first': u'كن',
        'second': u'كن'
    },
    u'كنهما': {
        'first': u'كن',
        'second': u'كن'
    },
    u'كنهم': {
        'first': u'كن',
        'second': u'كن'
    },
    u'كنهن': {
        'first': u'كن',
        'second': u'كن'
    },
    u'هي': {
        'first': u'ه',
        'second': u'ه'
    },
    u'هنا': {
        'first': u'ه',
        'second': u'ه'
    },
    u'هك': {
        'first': u'ه',
        'second': u'ه'
    },
    u'هكما': {
        'first': u'ه',
        'second': u'ه'
    },
    u'هكم': {
        'first': u'ه',
        'second': u'ه'
    },
    u'هكن': {
        'first': u'ه',
        'second': u'ه'
    },
    u'هه': {
        'first': u'ه',
        'second': u'ه'
    },
    u'هها': {
        'first': u'ه',
        'second': u'ه'
    },
    u'ههما': {
        'first': u'ه',
        'second': u'ه'
    },
    u'ههم': {
        'first': u'ه',
        'second': u'ه'
    },
    u'ههن': {
        'first': u'ه',
        'second': u'ه'
    },
    u'هاي': {
        'first': u'ها',
        'second': u'ها'
    },
    u'هاني': {
        'first': u'ها',
        'second': u'ها'
    },
    u'هانا': {
        'first': u'ها',
        'second': u'ها'
    },
    u'هاك': {
        'first': u'ها',
        'second': u'ها'
    },
    u'هاكما': {
        'first': u'ها',
        'second': u'ها'
    },
    u'هاكم': {
        'first': u'ها',
        'second': u'ها'
    },
    u'هاكن': {
        'first': u'ها',
        'second': u'ها'
    },
    u'هاه': {
        'first': u'ها',
        'second': u'ها'
    },
    u'هاها': {
        'first': u'ها',
        'second': u'ها'
    },
    u'هاهما': {
        'first': u'ها',
        'second': u'ها'
    },
    u'هاهم': {
        'first': u'ها',
        'second': u'ها'
    },
    u'هاهن': {
        'first': u'ها',
        'second': u'ها'
    },
    u'هماي': {
        'first': u'هما',
        'second': u'هما'
    },
    u'هماني': {
        'first': u'هما',
        'second': u'هما'
    },
    u'همانا': {
        'first': u'هما',
        'second': u'هما'
    },
    u'هماك': {
        'first': u'هما',
        'second': u'هما'
    },
    u'هماكما': {
        'first': u'هما',
        'second': u'هما'
    },
    u'هماكم': {
        'first': u'هما',
        'second': u'هما'
    },
    u'هماكن': {
        'first': u'هما',
        'second': u'هما'
    },
    u'هماه': {
        'first': u'هما',
        'second': u'هما'
    },
    u'هماها': {
        'first': u'هما',
        'second': u'هما'
    },
    u'هماهما': {
        'first': u'هما',
        'second': u'هما'
    },
    u'هماهم': {
        'first': u'هما',
        'second': u'هما'
    },
    u'هماهن': {
        'first': u'هما',
        'second': u'هما'
    },
    u'هموي': {
        'first': u'هم',
        'second': u'هم'
    },
    u'هموني': {
        'first': u'هم',
        'second': u'هم'
    },
    u'همونا': {
        'first': u'هم',
        'second': u'هم'
    },
    u'هموك': {
        'first': u'هم',
        'second': u'هم'
    },
    u'هموكما': {
        'first': u'هم',
        'second': u'هم'
    },
    u'هموكم': {
        'first': u'هم',
        'second': u'هم'
    },
    u'هموكن': {
        'first': u'هم',
        'second': u'هم'
    },
    u'هموه': {
        'first': u'هم',
        'second': u'هم'
    },
    u'هموها': {
        'first': u'هم',
        'second': u'هم'
    },
    u'هموهما': {
        'first': u'هم',
        'second': u'هم'
    },
    u'هموهم': {
        'first': u'هم',
        'second': u'هم'
    },
    u'هموهن': {
        'first': u'هم',
        'second': u'هم'
    },
    u'هني': {
        'first': u'هن',
        'second': u'هن'
    },
    u'هنني': {
        'first': u'هن',
        'second': u'هن'
    },
    u'هننا': {
        'first': u'هن',
        'second': u'هن'
    },
    u'هنك': {
        'first': u'هن',
        'second': u'هن'
    },
    u'هنكما': {
        'first': u'هن',
        'second': u'هن'
    },
    u'هنكم': {
        'first': u'هن',
        'second': u'هن'
    },
    u'هنكن': {
        'first': u'هن',
        'second': u'هن'
    },
    u'هنه': {
        'first': u'هن',
        'second': u'هن'
    },
    u'هنها': {
        'first': u'هن',
        'second': u'هن'
    },
    u'هنهما': {
        'first': u'هن',
        'second': u'هن'
    },
    u'هنهم': {
        'first': u'هن',
        'second': u'هن'
    },
    u'هنهن': {
        'first': u'هن',
        'second': u'هن'
    },
}
# created on fly at the program start, from the Table_affix
TABLE_AFFIX_INDEX = {
    '': {
        'pronouns':
        qutrubVerbConst.PronounsTable,
        'tenses': [
            qutrubVerbConst.TenseJussiveFuture,
            qutrubVerbConst.TensePassiveConfirmedFuture,
            qutrubVerbConst.TensePassiveFuture,
            qutrubVerbConst.TensePassiveJussiveFuture,
            qutrubVerbConst.TensePassivePast,
            qutrubVerbConst.TensePassiveSubjunctiveFuture,
            qutrubVerbConst.TensePast,
            qutrubVerbConst.TenseSubjunctiveFuture,
            qutrubVerbConst.TenseConfirmedFuture,
            qutrubVerbConst.TenseConfirmedImperative,
            qutrubVerbConst.TenseFuture,
            qutrubVerbConst.TenseImperative,
        ]
    },
}
TABLE_AFFIX = {
    u'ت-ون': [
        (u'المضارع المجهول', u'أنتم'),
        (u'المضارع المعلوم', u'أنتم'),
    ],
    u'-ن': [
        (u'الماضي المجهول', u'هن'),
        (u'الأمر المؤكد', u'أنتِ'),
        (u'الأمر المؤكد', u'أنت'),
        (u'الأمر المؤكد', u'أنتم'),
        (u'الماضي المعلوم', u'هن'),
        (u'الأمر', u'أنتن'),
    ],
    u'أ-': [
        (u'المضارع المنصوب', u'أنا'),
        (u'المضارع المجهول المجزوم', u'أنا'),
        (u'المضارع المجهول', u'أنا'),
        (u'المضارع المعلوم', u'أنا'),
        (u'المضارع المجزوم', u'أنا'),
        (u'المضارع المجهول المنصوب', u'أنا'),
    ],
    u'ت-نان': [
        (u'المضارع المؤكد الثقيل', u'أنتن'),
        (u'المضارع المؤكد الثقيل المجهول ', u'أنتن'),
    ],
    u'-ي': [
        (u'الأمر', u'أنتِ'),
    ],
    u'ت-ان': [
        (u'المضارع المؤكد الثقيل', u'أنتما مؤ'),
        (u'المضارع المؤكد الثقيل', u'أنتما'),
        (u'المضارع المؤكد الثقيل', u'هما مؤ'),
        (u'المضارع المجهول', u'أنتما مؤ'),
        (u'المضارع المجهول', u'أنتما'),
        (u'المضارع المجهول', u'هما مؤ'),
        (u'المضارع المعلوم', u'أنتما مؤ'),
        (u'المضارع المعلوم', u'أنتما'),
        (u'المضارع المعلوم', u'هما مؤ'),
        (u'المضارع المؤكد الثقيل المجهول ', u'أنتما مؤ'),
        (u'المضارع المؤكد الثقيل المجهول ', u'أنتما'),
        (u'المضارع المؤكد الثقيل المجهول ', u'هما مؤ'),
    ],
    u'ت-ا': [
        (u'المضارع المنصوب', u'أنتما مؤ'),
        (u'المضارع المنصوب', u'أنتما'),
        (u'المضارع المنصوب', u'هما مؤ'),
        (u'المضارع المجهول المجزوم', u'أنتما مؤ'),
        (u'المضارع المجهول المجزوم', u'أنتما'),
        (u'المضارع المجهول المجزوم', u'هما مؤ'),
        (u'المضارع المجزوم', u'أنتما مؤ'),
        (u'المضارع المجزوم', u'أنتما'),
        (u'المضارع المجزوم', u'هما مؤ'),
        (u'المضارع المجهول المنصوب', u'أنتما مؤ'),
        (u'المضارع المجهول المنصوب', u'أنتما'),
        (u'المضارع المجهول المنصوب', u'هما مؤ'),
    ],
    u'-وا': [
        (u'الماضي المجهول', u'هم'),
        (u'الماضي المعلوم', u'هم'),
        (u'الأمر', u'أنتم'),
    ],
    u'-تا': [
        (u'الماضي المجهول', u'هما مؤ'),
        (u'الماضي المعلوم', u'هما مؤ'),
    ],
    u'ي-ون': [
        (u'المضارع المجهول', u'هم'),
        (u'المضارع المعلوم', u'هم'),
    ],
    u'-': [
        (u'الماضي المجهول', u'هو'),
        (u'الماضي المعلوم', u'هو'),
        (u'الأمر', u'أنت'),
    ],
    u'ي-نان': [
        (u'المضارع المؤكد الثقيل', u'هن'),
        (u'المضارع المؤكد الثقيل المجهول ', u'هن'),
    ],
    u'ي-ا': [
        (u'المضارع المنصوب', u'هما'),
        (u'المضارع المجهول المجزوم', u'هما'),
        (u'المضارع المجزوم', u'هما'),
        (u'المضارع المجهول المنصوب', u'هما'),
    ],
    u'-تم': [
        (u'الماضي المجهول', u'أنتم'),
        (u'الماضي المعلوم', u'أنتم'),
    ],
    u'-تن': [
        (u'الماضي المجهول', u'أنتن'),
        (u'الماضي المعلوم', u'أنتن'),
    ],
    u'ي-ان': [
        (u'المضارع المؤكد الثقيل', u'هما'),
        (u'المضارع المجهول', u'هما'),
        (u'المضارع المعلوم', u'هما'),
        (u'المضارع المؤكد الثقيل المجهول ', u'هما'),
    ],
    u'ي-وا': [
        (u'المضارع المنصوب', u'هم'),
        (u'المضارع المجهول المجزوم', u'هم'),
        (u'المضارع المجزوم', u'هم'),
        (u'المضارع المجهول المنصوب', u'هم'),
    ],
    u'أ-ن': [
        (u'المضارع المؤكد الثقيل', u'أنا'),
        (u'المضارع المؤكد الثقيل المجهول ', u'أنا'),
    ],
    u'ت-': [
        (u'المضارع المنصوب', u'أنت'),
        (u'المضارع المنصوب', u'هي'),
        (u'المضارع المجهول المجزوم', u'أنت'),
        (u'المضارع المجهول المجزوم', u'هي'),
        (u'المضارع المجهول', u'أنت'),
        (u'المضارع المجهول', u'هي'),
        (u'المضارع المعلوم', u'أنت'),
        (u'المضارع المعلوم', u'هي'),
        (u'المضارع المجزوم', u'أنت'),
        (u'المضارع المجزوم', u'هي'),
        (u'المضارع المجهول المنصوب', u'أنت'),
        (u'المضارع المجهول المنصوب', u'هي'),
    ],
    u'ت-ين': [
        (u'المضارع المجهول', u'أنتِ'),
        (u'المضارع المعلوم', u'أنتِ'),
    ],
    u'ي-ن': [
        (u'المضارع المنصوب', u'هن'),
        (u'المضارع المؤكد الثقيل', u'هم'),
        (u'المضارع المؤكد الثقيل', u'هو'),
        (u'المضارع المجهول المجزوم', u'هن'),
        (u'المضارع المجهول', u'هن'),
        (u'المضارع المعلوم', u'هن'),
        (u'المضارع المجزوم', u'هن'),
        (u'المضارع المجهول المنصوب', u'هن'),
        (u'المضارع المؤكد الثقيل المجهول ', u'هم'),
        (u'المضارع المؤكد الثقيل المجهول ', u'هو'),
    ],
    u'-تما': [
        (u'الماضي المجهول', u'أنتما مؤ'),
        (u'الماضي المجهول', u'أنتما'),
        (u'الماضي المعلوم', u'أنتما مؤ'),
        (u'الماضي المعلوم', u'أنتما'),
    ],
    u'-ا': [
        (u'الماضي المجهول', u'هما'),
        (u'الماضي المعلوم', u'هما'),
        (u'الأمر', u'أنتما مؤ'),
        (u'الأمر', u'أنتما'),
    ],
    u'-ان': [
        (u'الأمر المؤكد', u'أنتما مؤ'),
        (u'الأمر المؤكد', u'أنتما'),
    ],
    u'ت-وا': [
        (u'المضارع المنصوب', u'أنتم'),
        (u'المضارع المجهول المجزوم', u'أنتم'),
        (u'المضارع المجزوم', u'أنتم'),
        (u'المضارع المجهول المنصوب', u'أنتم'),
    ],
    u'-نا': [
        (u'الماضي المجهول', u'نحن'),
        (u'الماضي المعلوم', u'نحن'),
    ],
    u'-نان': [
        (u'الأمر المؤكد', u'أنتن'),
    ],
    u'-ت': [
        (u'الماضي المجهول', u'أنتِ'),
        (u'الماضي المجهول', u'أنت'),
        (u'الماضي المجهول', u'أنا'),
        (u'الماضي المجهول', u'هي'),
        (u'الماضي المعلوم', u'أنتِ'),
        (u'الماضي المعلوم', u'أنت'),
        (u'الماضي المعلوم', u'أنا'),
        (u'الماضي المعلوم', u'هي'),
    ],
    u'ت-ي': [
        (u'المضارع المنصوب', u'أنتِ'),
        (u'المضارع المجهول المجزوم', u'أنتِ'),
        (u'المضارع المجزوم', u'أنتِ'),
        (u'المضارع المجهول المنصوب', u'أنتِ'),
    ],
    u'ي-': [
        (u'المضارع المنصوب', u'هو'),
        (u'المضارع المجهول المجزوم', u'هو'),
        (u'المضارع المجهول', u'هو'),
        (u'المضارع المعلوم', u'هو'),
        (u'المضارع المجزوم', u'هو'),
        (u'المضارع المجهول المنصوب', u'هو'),
    ],
    u'ن-ن': [
        (u'المضارع المؤكد الثقيل', u'نحن'),
        (u'المضارع المؤكد الثقيل المجهول ', u'نحن'),
    ],
    u'ت-ن': [
        (u'المضارع المنصوب', u'أنتن'),
        (u'المضارع المؤكد الثقيل', u'أنتِ'),
        (u'المضارع المؤكد الثقيل', u'أنت'),
        (u'المضارع المؤكد الثقيل', u'أنتم'),
        (u'المضارع المؤكد الثقيل', u'هي'),
        (u'المضارع المجهول المجزوم', u'أنتن'),
        (u'المضارع المجهول', u'أنتن'),
        (u'المضارع المعلوم', u'أنتن'),
        (u'المضارع المجزوم', u'أنتن'),
        (u'المضارع المجهول المنصوب', u'أنتن'),
        (u'المضارع المؤكد الثقيل المجهول ', u'أنتِ'),
        (u'المضارع المؤكد الثقيل المجهول ', u'أنت'),
        (u'المضارع المؤكد الثقيل المجهول ', u'أنتم'),
        (u'المضارع المؤكد الثقيل المجهول ', u'هي'),
    ],
    u'ن-': [
        (u'المضارع المنصوب', u'نحن'),
        (u'المضارع المجهول المجزوم', u'نحن'),
        (u'المضارع المجهول', u'نحن'),
        (u'المضارع المعلوم', u'نحن'),
        (u'المضارع المجزوم', u'نحن'),
        (u'المضارع المجهول المنصوب', u'نحن'),
    ],
}
EXTERNAL_PREFIX_TABLE = {}
# [ أ الإستفهام]
EXTERNAL_PREFIX_TABLE[u'أ'] = set([
    qutrubVerbConst.TenseConfirmedFuture,
    # qutrubVerbConst.TenseConfirmedImperative,
    qutrubVerbConst.TenseFuture,
    # qutrubVerbConst.TenseImperative,
    # qutrubVerbConst.TenseJussiveFuture,
    qutrubVerbConst.TensePassiveConfirmedFuture,
    qutrubVerbConst.TensePassiveFuture,
    # qutrubVerbConst.TensePassiveJussiveFuture,
    qutrubVerbConst.TensePassivePast,
    # qutrubVerbConst.TensePassiveSubjunctiveFuture,
    qutrubVerbConst.TensePast,
    qutrubVerbConst.TenseSubjunctiveFuture,
])
# [و العطف]
EXTERNAL_PREFIX_TABLE[u'و'] = set([
    qutrubVerbConst.TenseConfirmedFuture,
    qutrubVerbConst.TenseConfirmedImperative,
    qutrubVerbConst.TenseFuture,
    qutrubVerbConst.TenseImperative,
    qutrubVerbConst.TenseJussiveFuture,
    qutrubVerbConst.TensePassiveConfirmedFuture,
    qutrubVerbConst.TensePassiveFuture,
    qutrubVerbConst.TensePassiveJussiveFuture,
    qutrubVerbConst.TensePassivePast,
    qutrubVerbConst.TensePassiveSubjunctiveFuture,
    qutrubVerbConst.TensePast,
    qutrubVerbConst.TenseSubjunctiveFuture,
])
# [ أ الإستفهام][و العطف]
EXTERNAL_PREFIX_TABLE[u'أو'] = set([
    qutrubVerbConst.TenseConfirmedFuture,
    # qutrubVerbConst.TenseConfirmedImperative,
    qutrubVerbConst.TenseFuture,
    # qutrubVerbConst.TenseImperative,
    # qutrubVerbConst.TenseJussiveFuture,
    qutrubVerbConst.TensePassiveConfirmedFuture,
    # qutrubVerbConst.TensePassiveFuture,
    # qutrubVerbConst.TensePassiveJussiveFuture,
    qutrubVerbConst.TensePassivePast,
    # qutrubVerbConst.TensePassiveSubjunctiveFuture,
    qutrubVerbConst.TensePast,
    # qutrubVerbConst.TenseSubjunctiveFuture,
])
# [ف العطف]
EXTERNAL_PREFIX_TABLE[u'ف'] = set([
    qutrubVerbConst.TenseConfirmedFuture,
    qutrubVerbConst.TenseConfirmedImperative,
    qutrubVerbConst.TenseFuture,
    qutrubVerbConst.TenseImperative,
    qutrubVerbConst.TenseJussiveFuture,
    qutrubVerbConst.TensePassiveConfirmedFuture,
    qutrubVerbConst.TensePassiveFuture,
    qutrubVerbConst.TensePassiveJussiveFuture,
    qutrubVerbConst.TensePassivePast,
    qutrubVerbConst.TensePassiveSubjunctiveFuture,
    qutrubVerbConst.TensePast,
    qutrubVerbConst.TenseSubjunctiveFuture,
])
# [ف السببيّة]
#Added to Feh AlAtf
EXTERNAL_PREFIX_TABLE[u'ف'] |= set([
    qutrubVerbConst.TensePassiveSubjunctiveFuture,
    qutrubVerbConst.TenseSubjunctiveFuture,
])
# [ أ الإستفهام][ف العطف]
EXTERNAL_PREFIX_TABLE[u'أف'] = set([
    qutrubVerbConst.TenseConfirmedFuture,
    # qutrubVerbConst.TenseConfirmedImperative,
    qutrubVerbConst.TenseFuture,
    # qutrubVerbConst.TenseImperative,
    # qutrubVerbConst.TenseJussiveFuture,
    qutrubVerbConst.TensePassiveConfirmedFuture,
    # qutrubVerbConst.TensePassiveFuture,
    # qutrubVerbConst.TensePassiveJussiveFuture,
    qutrubVerbConst.TensePassivePast,
    # qutrubVerbConst.TensePassiveSubjunctiveFuture,
    qutrubVerbConst.TensePast,
    # qutrubVerbConst.TenseSubjunctiveFuture,
])
# [ل التوكيد]
EXTERNAL_PREFIX_TABLE[u'ل'] = set([
    qutrubVerbConst.TenseConfirmedFuture,
    # qutrubVerbConst.TenseConfirmedImperative,
    qutrubVerbConst.TenseFuture,
    # qutrubVerbConst.TenseImperative,
    # qutrubVerbConst.TenseJussiveFuture,
    qutrubVerbConst.TensePassiveConfirmedFuture,
    # qutrubVerbConst.TensePassiveFuture,
    # qutrubVerbConst.TensePassiveJussiveFuture,
    qutrubVerbConst.TensePassivePast,
    # qutrubVerbConst.TensePassiveSubjunctiveFuture,
    qutrubVerbConst.TensePast,
    # qutrubVerbConst.TenseSubjunctiveFuture,
])
# [و العطف][ل التوكيد]
EXTERNAL_PREFIX_TABLE[u'ول'] = set([
    qutrubVerbConst.TenseConfirmedFuture,
    # qutrubVerbConst.TenseConfirmedImperative,
    qutrubVerbConst.TenseFuture,
    # qutrubVerbConst.TenseImperative,
    # qutrubVerbConst.TenseJussiveFuture,
    qutrubVerbConst.TensePassiveConfirmedFuture,
    # qutrubVerbConst.TensePassiveFuture,
    # qutrubVerbConst.TensePassiveJussiveFuture,
    qutrubVerbConst.TensePassivePast,
    # qutrubVerbConst.TensePassiveSubjunctiveFuture,
    qutrubVerbConst.TensePast,
    # qutrubVerbConst.TenseSubjunctiveFuture,
])
# [ف العطف][ل التوكيد]
EXTERNAL_PREFIX_TABLE[u'فل'] = set([
    qutrubVerbConst.TenseConfirmedFuture,
    # qutrubVerbConst.TenseConfirmedImperative,
    qutrubVerbConst.TenseFuture,
    # qutrubVerbConst.TenseImperative,
    # qutrubVerbConst.TenseJussiveFuture,
    qutrubVerbConst.TensePassiveConfirmedFuture,
    # qutrubVerbConst.TensePassiveFuture,
    # qutrubVerbConst.TensePassiveJussiveFuture,
    qutrubVerbConst.TensePassivePast,
    # qutrubVerbConst.TensePassiveSubjunctiveFuture,
    qutrubVerbConst.TensePast,
    # qutrubVerbConst.TenseSubjunctiveFuture,
])
# [ل التعليل]
# added to لام التوكيد
EXTERNAL_PREFIX_TABLE[u'ل'] |= set([
    qutrubVerbConst.TensePassiveSubjunctiveFuture,
    qutrubVerbConst.TenseSubjunctiveFuture,
])
# [ أ الإستفهام][ل التعليل]
EXTERNAL_PREFIX_TABLE[u'أل'] = set([
    qutrubVerbConst.TensePassiveSubjunctiveFuture,
    qutrubVerbConst.TenseSubjunctiveFuture,
])
# [و العطف][ل التعليل]
# added to لام التوكيد
EXTERNAL_PREFIX_TABLE[u'ول'] |= set([
    qutrubVerbConst.TensePassiveSubjunctiveFuture,
    qutrubVerbConst.TenseSubjunctiveFuture,
])
# [ف العطف][ل التعليل]
# added to لام التوكيد
EXTERNAL_PREFIX_TABLE[u'فل'] |= set([
    qutrubVerbConst.TensePassiveSubjunctiveFuture,
    qutrubVerbConst.TenseSubjunctiveFuture,
])
# [ أ الإستفهام][و العطف][ل التعليل]
EXTERNAL_PREFIX_TABLE[u'أول'] = set([
    qutrubVerbConst.TensePassiveSubjunctiveFuture,
    qutrubVerbConst.TenseSubjunctiveFuture,
])
# [ أ الإستفهام][ف العطف][ل التعليل]
EXTERNAL_PREFIX_TABLE[u'أفل'] = set([
    qutrubVerbConst.TensePassiveSubjunctiveFuture,
    qutrubVerbConst.TenseSubjunctiveFuture,
])
# [ل الأمر]
#added to Lam Tawkid
EXTERNAL_PREFIX_TABLE[u'ل'] |= set([
    qutrubVerbConst.TenseJussiveFuture,
    qutrubVerbConst.TensePassiveJussiveFuture,
])
# [و العطف][ل الأمر]
#added to Lam Tawkid
EXTERNAL_PREFIX_TABLE[u'ول'] |= set([
    qutrubVerbConst.TenseJussiveFuture,
    qutrubVerbConst.TensePassiveJussiveFuture,
])
# [ف العطف][ل الأمر]
#added to Lam Tawkid
EXTERNAL_PREFIX_TABLE[u'فل'] |= set([
    qutrubVerbConst.TenseJussiveFuture,
    qutrubVerbConst.TensePassiveJussiveFuture,
])
# [س سوف]
EXTERNAL_PREFIX_TABLE[u'س'] = set([
    qutrubVerbConst.TenseFuture,
    qutrubVerbConst.TensePassiveFuture,
])
# [ أ الإستفهام][س سوف]
EXTERNAL_PREFIX_TABLE[u'أس'] = set([
    qutrubVerbConst.TenseFuture,
    qutrubVerbConst.TensePassiveFuture,
])
# [و العطف][س سوف]
EXTERNAL_PREFIX_TABLE[u'وس'] = set([
    qutrubVerbConst.TenseFuture,
    qutrubVerbConst.TensePassiveFuture,
])
# [ف العطف][س سوف]
EXTERNAL_PREFIX_TABLE[u'فس'] = set([
    qutrubVerbConst.TenseFuture,
    qutrubVerbConst.TensePassiveFuture,
])
# [ أ الإستفهام][و العطف][س سوف]
EXTERNAL_PREFIX_TABLE[u'أوس'] = set([
    qutrubVerbConst.TenseFuture,
    qutrubVerbConst.TensePassiveFuture,
])
# [ أ الإستفهام][ف العطف][س سوف]
EXTERNAL_PREFIX_TABLE[u'أفس'] = set([
    qutrubVerbConst.TenseFuture,
    qutrubVerbConst.TensePassiveFuture,
])
EXTERNAL_SUFFIX_TABLE = {}
# تصلح فقط مع الأفعال الخمسة
#ToDo
# التحقق من يالء مع ا\لأفعال  الخمسة فقط
EXTERNAL_SUFFIX_TABLE[u'ي'] = (
    # qutrubVerbConst.PronounAna, #u"أنا";
    # qutrubVerbConst.PronounNahnu,#u"نحن";
    # qutrubVerbConst.PronounAnta,   #u"أنت";
    qutrubVerbConst.PronounAnti,  #u"أنتِ";
    qutrubVerbConst.PronounAntuma,  #u"أنتما";
    qutrubVerbConst.PronounAntuma_f,  #u"أنتما مؤ";
    qutrubVerbConst.PronounAntum,  #u"أنتم";
    # qutrubVerbConst.PronounAntunna,   #u"أنتن";
    # qutrubVerbConst.PronounHuwa,   #u"هو";
    # qutrubVerbConst.PronounHya,   #u"هي";
    # qutrubVerbConst.PronounHuma,   #u"هما";
    # qutrubVerbConst.PronounHuma_f,   #u"هما مؤ";
    # qutrubVerbConst.PronounHum,   #u"هم";
    # qutrubVerbConst.PronounHunna,   #u"هن";
)
EXTERNAL_SUFFIX_TABLE[u'ني'] = (
    # أفعال القلوب المتعدية لمفعول به عاقل
    qutrubVerbConst.PronounAna,  #u"أنا";
    # qutrubVerbConst.PronounNahnu,#u"نحن";
    qutrubVerbConst.PronounAnta,  #u"أنت";
    qutrubVerbConst.PronounAnti,  #u"أنتِ";
    qutrubVerbConst.PronounAntuma,  #u"أنتما";
    qutrubVerbConst.PronounAntuma_f,  #u"أنتما مؤ";
    qutrubVerbConst.PronounAntum,  #u"أنتم";
    qutrubVerbConst.PronounAntunna,  #u"أنتن";
    qutrubVerbConst.PronounHuwa,  #u"هو";
    qutrubVerbConst.PronounHya,  #u"هي";
    qutrubVerbConst.PronounHuma,  #u"هما";
    qutrubVerbConst.PronounHuma_f,  #u"هما مؤ";
    qutrubVerbConst.PronounHum,  #u"هم";
    qutrubVerbConst.PronounHunna,  #u"هن";
)
EXTERNAL_SUFFIX_TABLE[u'نا'] = (
    # أفعال القلوب المتعدية لمفعول به عاقل
    qutrubVerbConst.PronounAna,  #u"أنا";
    # أفعال القلوب المتعدية لمفعول به عاقل
    qutrubVerbConst.PronounNahnu,  #u"نحن";
    qutrubVerbConst.PronounAnta,  #u"أنت";
    qutrubVerbConst.PronounAnti,  #u"أنتِ";
    qutrubVerbConst.PronounAntuma,  #u"أنتما";
    qutrubVerbConst.PronounAntuma_f,  #u"أنتما مؤ";
    qutrubVerbConst.PronounAntum,  #u"أنتم";
    qutrubVerbConst.PronounAntunna,  #u"أنتن";
    qutrubVerbConst.PronounHuwa,  #u"هو";
    qutrubVerbConst.PronounHya,  #u"هي";
    qutrubVerbConst.PronounHuma,  #u"هما";
    qutrubVerbConst.PronounHuma_f,  #u"هما مؤ";
    qutrubVerbConst.PronounHum,  #u"هم";
    qutrubVerbConst.PronounHunna,  #u"هن";
)
# للمذكر: يحذف صمير المؤنث
# للمؤنث يحذف صمير المذكر
EXTERNAL_SUFFIX_TABLE[u'ك'] = (
    qutrubVerbConst.PronounAna,  #u"أنا";
    qutrubVerbConst.PronounNahnu,  #u"نحن";
    # أفعال القلوب المتعدية لمفعول به عاقل
    qutrubVerbConst.PronounAnta,  #u"أنت";
    # أفعال القلوب المتعدية لمفعول به عاقل
    qutrubVerbConst.PronounAnti,  #u"أنتِ";
    # qutrubVerbConst.PronounAntuma,   #u"أنتما";
    # qutrubVerbConst.PronounAntuma_f,   #u"أنتما مؤ";
    # qutrubVerbConst.PronounAntum,   #u"أنتم";
    # qutrubVerbConst.PronounAntunna,   #u"أنتن";
    qutrubVerbConst.PronounHuwa,  #u"هو";
    qutrubVerbConst.PronounHya,  #u"هي";
    qutrubVerbConst.PronounHuma,  #u"هما";
    qutrubVerbConst.PronounHuma_f,  #u"هما مؤ";
    qutrubVerbConst.PronounHum,  #u"هم";
    qutrubVerbConst.PronounHunna,  #u"هن";
)
EXTERNAL_SUFFIX_TABLE[u'كما'] = (
    qutrubVerbConst.PronounAna,  #u"أنا";
    qutrubVerbConst.PronounNahnu,  #u"نحن";
    # أفعال القلوب المتعدية لمفعول به عاقل
    qutrubVerbConst.PronounAnta,  #u"أنت";
    # أفعال القلوب المتعدية لمفعول به عاقل
    qutrubVerbConst.PronounAnti,  #u"أنتِ";
    # أفعال القلوب المتعدية لمفعول به عاقل
    qutrubVerbConst.PronounAntuma,  #u"أنتما";
    # أفعال القلوب المتعدية لمفعول به عاقل
    qutrubVerbConst.PronounAntuma_f,  #u"أنتما مؤ";
    # qutrubVerbConst.PronounAntum,   #u"أنتم";
    # qutrubVerbConst.PronounAntunna,   #u"أنتن";
    qutrubVerbConst.PronounHuwa,  #u"هو";
    qutrubVerbConst.PronounHya,  #u"هي";
    qutrubVerbConst.PronounHuma,  #u"هما";
    qutrubVerbConst.PronounHuma_f,  #u"هما مؤ";
    qutrubVerbConst.PronounHum,  #u"هم";
    qutrubVerbConst.PronounHunna,  #u"هن";
)
EXTERNAL_SUFFIX_TABLE[u'كم'] = (
    qutrubVerbConst.PronounAna,  #u"أنا";
    qutrubVerbConst.PronounNahnu,  #u"نحن";
    # أفعال القلوب المتعدية لمفعول به عاقل
    qutrubVerbConst.PronounAnta,  #u"أنت";
    # أفعال القلوب المتعدية لمفعول به عاقل
    qutrubVerbConst.PronounAnti,  #u"أنتِ";
    # أفعال القلوب المتعدية لمفعول به عاقل
    qutrubVerbConst.PronounAntuma,  #u"أنتما";
    # أفعال القلوب المتعدية لمفعول به عاقل
    qutrubVerbConst.PronounAntuma_f,  #u"أنتما مؤ";
    # أفعال القلوب المتعدية لمفعول به عاقل
    qutrubVerbConst.PronounAntum,  #u"أنتم";
    # أفعال القلوب المتعدية لمفعول به عاقل
    qutrubVerbConst.PronounAntunna,  #u"أنتن";
    qutrubVerbConst.PronounHuwa,  #u"هو";
    qutrubVerbConst.PronounHya,  #u"هي";
    qutrubVerbConst.PronounHuma,  #u"هما";
    qutrubVerbConst.PronounHuma_f,  #u"هما مؤ";
    qutrubVerbConst.PronounHum,  #u"هم";
    qutrubVerbConst.PronounHunna,  #u"هن";
)
EXTERNAL_SUFFIX_TABLE[u'كن'] = (
    qutrubVerbConst.PronounAna,  #u"أنا";
    qutrubVerbConst.PronounNahnu,  #u"نحن";
    # qutrubVerbConst.PronounAnta,   #u"أنت";
    # أفعال القلوب المتعدية لمفعول به عاقل
    qutrubVerbConst.PronounAnti,  #u"أنتِ";
    qutrubVerbConst.PronounAntuma,  #u"أنتما";
    # أفعال القلوب المتعدية لمفعول به عاقل
    qutrubVerbConst.PronounAntuma_f,  #u"أنتما مؤ";
    # qutrubVerbConst.PronounAntum,   #u"أنتم";
    # أفعال القلوب المتعدية لمفعول به عاقل
    qutrubVerbConst.PronounAntunna,  #u"أنتن";
    qutrubVerbConst.PronounHuwa,  #u"هو";
    qutrubVerbConst.PronounHya,  #u"هي";
    qutrubVerbConst.PronounHuma,  #u"هما";
    qutrubVerbConst.PronounHuma_f,  #u"هما مؤ";
    qutrubVerbConst.PronounHum,  #u"هم";
    qutrubVerbConst.PronounHunna,  #u"هن";
)
# This cases take all Pronoun
EXTERNAL_SUFFIX_TABLE[u'ه'] = qutrubVerbConst.PronounsTable
EXTERNAL_SUFFIX_TABLE[u'ها'] = qutrubVerbConst.PronounsTable
EXTERNAL_SUFFIX_TABLE[u'هما'] = qutrubVerbConst.PronounsTable
EXTERNAL_SUFFIX_TABLE[u'هم'] = qutrubVerbConst.PronounsTable
EXTERNAL_SUFFIX_TABLE[u'هنّ'] = qutrubVerbConst.PronounsTable
EXTERNAL_SUFFIX_TABLE[u'هن'] = qutrubVerbConst.PronounsTable



#!/usr/bin/python
# -*- coding=utf-8 -*-
#-------------------------------------------------------------------------
# Name:        stem_stop
# Purpose:     Arabic lexical analyser, provides feature for
#~stemming arabic word as stop
#
# Author:      Taha Zerrouki (taha.zerrouki[at]gmail.com)
#
# Created:     31-10-2011
# Copyright:   (c) Taha Zerrouki 2011
# Licence:     GPL
#-------------------------------------------------------------------------
"""
    Arabic stop stemmer
"""
#~ import re
import pyarabic.araby as araby
import tashaphyne.stemming
import tashaphyne.normalize
from . import stem_stopwords_const as ssconst
import arramooz.stopwordsdictionaryclass as stopwordsdictionaryclass
from . import wordcase


class StopWordStemmer:
    """
        Arabic stop stemmer
    """

    def __init__(self, debug=False):
        # create a stemmer object for stemming enclitics and procletics
        self.comp_stemmer = tashaphyne.stemming.ArabicLightStemmer()
        # configure the stemmer object
        self.comp_stemmer.set_prefix_list(ssconst.COMP_PREFIX_LIST)
        self.comp_stemmer.set_suffix_list(ssconst.COMP_SUFFIX_LIST)
        # create a stemmer object for stemming conjugated verb
        self.conj_stemmer = tashaphyne.stemming.ArabicLightStemmer()
        # configure the stemmer object
        self.conj_stemmer.set_prefix_list(ssconst.CONJ_PREFIX_LIST)
        self.conj_stemmer.set_suffix_list(ssconst.CONJ_SUFFIX_LIST)

        # enable the last mark (Harakat Al-I3rab)
        self.allow_syntax_lastmark = True

        # stop dictionary
        #~self.stop_dictionary = stopwordsdictionaryclass.StopWordsDictionary("stopwords")
        self.stop_dictionary = stopwordsdictionaryclass.StopWordsDictionary(
            "classedstopwords")

        # allow to print internal results.
        self.cache_dict_search = {}
        self.cache_affixes_verification = {}
        self.debug = debug

    def stemming_stopword(self, stop):
        """
        Analyze word morphologically as stop
        @param stop: the input stop.
        @type stop: unicode.
        @return: list of dictionaries of analyzed words with tags.
        @rtype: list.
        """
        #~list_found = []
        detailed_result = []
        stop_list = [
            stop,
        ]
        # if the word contains ALEF8MADDA, we convert it into 2 HAMZA above ALEF
        if araby.ALEF_MADDA in stop:
            stop_list.append(
                stop.replace(araby.ALEF_MADDA, araby.ALEF_HAMZA_ABOVE * 2))

        for stop in stop_list:
            list_seg_comp = self.comp_stemmer.segment(stop)
            list_seg_comp = self.verify_affix(stop, list_seg_comp,
                                         ssconst.COMP_STOPWORDS_AFFIXES)
            # treat multi vocalization encletic
            #~list_seg_comp_voc = []
            for seg in list_seg_comp:
                procletic = stop[:seg[0]]
                stem = stop[seg[0]:seg[1]]
                encletic_nm = stop[seg[1]:]

                # ajusting stops variant
                list_stem = [stem]
                if encletic_nm:  #  != ""
                    if stem.endswith(araby.ALEF):
                        list_stem.append(stem[:-1] + araby.ALEF_MAKSURA)
                    elif stem.endswith(araby.YEH):
                        list_stem.append(stem[:-1] + araby.ALEF_MAKSURA)
                    elif stem.endswith(araby.TEH):
                        list_stem.append(stem[:-1] + araby.TEH_MARBUTA)
                # treat gemination cases
                    if encletic_nm.startswith(araby.YEH):
                        list_stem.append(stem + araby.YEH)
                    elif encletic_nm.startswith(araby.NOON):
                        list_stem.append(stem + araby.NOON)

        # stem reduced stop : level two
                for stem in list_stem:
                    detailed_result.extend(
                        self.steming_second_level(stop, stem, procletic,
                                                  encletic_nm))
        return detailed_result  #list_found

    def steming_second_level(self, stop, stop2, procletic, encletic_nm):
        """
        Analyze word morphologically by stemming the conjugation affixes.
        @param stop: the input stop.
        @type stop: unicode.
        @param stop2: the stop stemed from syntaxic affixes.
        @type stop2: unicode.
        @param procletic: the syntaxic prefixe extracted in the fisrt stage.
        @type procletic: unicode.
        @param encletic_nm: the syntaxic suffixe extracted in the
        first stage (not vocalized).
        @type encletic_nm: unicode.
        @return: list of dictionaries of analyzed words with tags.
        @rtype: list.
        """
        detailed_result = []
        #segment the coinjugated verb
        list_seg_conj = self.conj_stemmer.segment(stop2)
        # verify affix compatibility
        list_seg_conj = self.verify_affix(stop2, list_seg_conj,
                                     ssconst.STOPWORDS_CONJUGATION_AFFIX)
        # add vocalized forms of suffixes
        # and create the real affixes from the word
        #~list_seg_conj_voc = []
        for seg_conj in list_seg_conj:
            stem_conj = stop2[seg_conj[0]:seg_conj[1]]
            suffix_conj_nm = stop2[seg_conj[1]:]

            # noirmalize hamza before gessing  differents origines
            #~stem_conj = araby.normalize_hamza(stem_conj)

            # generate possible stems
            # add stripped letters to the stem to constitute possible stop list
            possible_stop_list = self.get_stem_variants(stem_conj, suffix_conj_nm)

            # search the stop in the dictionary
            # we can return the tashkeel
            infstop_form_list = []
            for infstop in set(possible_stop_list):
                # get the stop and get all its forms from the dict
                # if the stop has plural suffix, don't look up in
                #broken plural dictionary
                if infstop not in self.cache_dict_search:
                    infstop_foundlist = self.stop_dictionary.lookup(infstop)
                    self.cache_dict_search[infstop] = self.create_dict_word(
                        infstop_foundlist)
                else:
                    infstop_foundlist = self.cache_dict_search[infstop]
                infstop_form_list.extend(infstop_foundlist)
            for stop_tuple in infstop_form_list:
                # stop_tuple = self.stop_dictionary.getEntryById(id)
                original = stop_tuple['vocalized']

                #test if the  given word from dictionary accept those
                # tags given by affixes
                # دراسة توافق الزوائد مع خصائص الاسم،
                # مثلا هل يقبل الاسم التأنيث.
                #~if validate_tags(stop_tuple, affix_tags, procletic, encletic_nm, suffix_conj_nm):
                for vocalized_encletic in ssconst.COMP_SUFFIX_LIST_TAGS[
                        encletic_nm]['vocalized']:
                    for vocalized_suffix in ssconst.CONJ_SUFFIX_LIST_TAGS[
                            suffix_conj_nm]['vocalized']:
                        # affixes tags contains prefixes and suffixes tags
                        affix_tags = ssconst.COMP_PREFIX_LIST_TAGS[procletic]['tags'] \
                                  +ssconst.COMP_SUFFIX_LIST_TAGS[vocalized_encletic]['tags'] \
                                  +ssconst.CONJ_SUFFIX_LIST_TAGS[vocalized_suffix]['tags']
                        ## verify compatibility between procletics and affix
                        valid = self.validate_tags(stop_tuple, affix_tags, procletic, encletic_nm)
                        compatible = self.is_compatible_proaffix_affix(
                            stop_tuple, procletic, vocalized_encletic,
                            vocalized_suffix)
                        if valid and compatible:
                            vocalized, semi_vocalized = self.vocalize(
                                original, procletic, vocalized_suffix,
                                vocalized_encletic)
                            vocalized = self.ajust_vocalization(vocalized)
                            #ToDo:
                            # if the stop word is inflected or not
                            is_inflected = u"مبني" if stop_tuple[
                                'is_inflected'] == 0 else u"معرب"
                            #add some tags from dictionary entry as
                            # use action and object_type
                            original_tags = u":".join([
                                stop_tuple['word_type'],
                                stop_tuple['word_class'],
                                is_inflected,
                                stop_tuple['action'],
                            ])
                            #~print "STOP_TUPEL[action]:", stop_tuple['action'].encode("utf8")
                            # generate word case
                            detailed_result.append(
                                wordcase.WordCase({
                                    'word':
                                    stop,
                                    'affix': (procletic, '', vocalized_suffix,
                                              vocalized_encletic),
                                    'stem':
                                    stem_conj,
                                    'original':
                                    original,
                                    'vocalized':
                                    vocalized,
                                    'semivocalized':
                                    semi_vocalized,
                                    'tags':
                                    u':'.join(affix_tags),
                                    'type':
                                    u':'.join(
                                        ['STOPWORD', stop_tuple['word_type']]),
                                    'freq':
                                    'freqstopword',  # to note the frequency type
                                    'originaltags':
                                    original_tags,
                                    "action":
                                    stop_tuple['action'],
                                    "object_type":
                                    stop_tuple['object_type'],
                                    "need":
                                    stop_tuple['need'],
                                    'syntax':
                                    '',
                                }))
        return detailed_result

    def is_compatible_proaffix_affix(self, stop_tuple, procletic, encletic,
                                     suffix):
        """
        Verify if proaffixes (sytaxic affixes) are compatable
        with affixes ( conjugation)
        @param procletic: first level prefix.
        @type procletic: unicode.
        @param encletic: first level suffix.
        @type encletic: unicode.
        @param suffix: second level suffix.
        @type suffix: unicode.
        @return: compatible.
        @rtype: True/False.
        """
        #if not procletic and not encletic:  return True
        #use cache for affix verification
        affix = u'-'.join([procletic, encletic, suffix])
        if affix in self.cache_affixes_verification:
            return self.cache_affixes_verification[affix]

        # get procletics and enclitics tags
        procletic_tags = ssconst.COMP_PREFIX_LIST_TAGS[procletic]['tags']
        encletic_tags = ssconst.COMP_SUFFIX_LIST_TAGS[encletic]['tags']
        # in stops there is no prefix
        suffix_tags = ssconst.CONJ_SUFFIX_LIST_TAGS[suffix]['tags']

        if u"تعريف" in procletic_tags and u"مضاف" in suffix_tags:
            self.cache_affixes_verification[affix] = False
        elif u"تعريف" in procletic_tags and u"تنوين" in suffix_tags:
            self.cache_affixes_verification[affix] = False

    # الجر  في حالات الاسم المعرفة بال أو الإضافة إلى ضمير أو مضاف إليه
    # مما يعني لا يمكن تطبيقها هنا
        elif u"مضاف" in encletic_tags and u"تنوين" in suffix_tags:
            self.cache_affixes_verification[affix] = False
        elif u"مضاف" in encletic_tags and u"لايضاف" in suffix_tags:
            self.cache_affixes_verification[affix] = False
        #~elif u"جر" in procletic_tags and u"مجرور" not in suffix_tags:
        #~self.cache_affixes_verification[affix] = False


#ستعمل في حالة كسر هاء الضمير في الجر

        elif  bool(u"لايجر" in encletic_tags) and  bool(u"مجرور" in \
        suffix_tags):
            self.cache_affixes_verification[affix] = False
        elif  bool(u"مجرور" in encletic_tags) and  not bool(u"جر" in \
        stop_tuple['word_class']):
            self.cache_affixes_verification[affix] = False
        elif  bool(u"مجرور" in encletic_tags) and  not bool(u"مجرور" in \
        suffix_tags):
            self.cache_affixes_verification[affix] = False

        else:
            self.cache_affixes_verification[affix] = True

        return self.cache_affixes_verification[affix]

    def set_debug(self, debug):
        """
        Set the debug attribute to allow printing internal analysis results.
        @param debug: the debug value.
        @type debug: True/False.
        """
        self.debug = debug

    def enable_syntax_lastmark(self):
        """
        Enable the syntaxic last mark attribute to allow use of I'rab harakat.
        """
        self.allow_syntax_lastmark = True

    def disable_syntax_lastmark(self):
        """
        Disable the syntaxic last mark attribute to allow use of I'rab harakat.
        """
        self.allow_syntax_lastmark = False

    @staticmethod
    def get_stem_variants(stem, suffix_nm):
        """
        Generate the Stop stem variants according to the affixes.
        For example مدرستي = >مدرست+ي = > مدرسة +ي.
        Return a list of possible cases.
        @param stem: the input stem.
        @type stem: unicode.
        @param suffix_nm: suffix (no mark).
        @type suffix_nm: unicode.
        @return: list of stem variants.
        @rtype: list of unicode.
        """
        #some cases must have some correction
        #determinate the  suffix types
        #~suffix = suffix_nm

        possible_stop_list = set([
            stem,
        ])
        if not suffix_nm or suffix_nm in (araby.YEH + araby.NOON,
                                          araby.WAW + araby.NOON):
            possible_stop = stem + araby.YEH
            possible_stop_list.add(possible_stop)
        if stem.endswith(araby.YEH):
            possible_stop = stem[:-1] + araby.ALEF_MAKSURA
            possible_stop_list.add(possible_stop)
        #to be validated
        validated_list = possible_stop_list
        return validated_list

    @staticmethod
    def get_suffix_variants(word, suffix, enclitic):
        """
        Get the suffix variant to be joined to the word.
        For example: word = مدرس, suffix = ة, encletic = ي.
        The suffix is converted to Teh.
        @param word: word found in dictionary.
        @type word: unicode.
        @param suffix: second level suffix.
        @type suffix: unicode.
        @param enclitic: first level suffix.
        @type enclitic: unicode.
        @return: variant of suffixes  (vocalized suffix and vocalized
        suffix without I'rab short mark).
        @rtype: (unicode, unicode)
        """
        enclitic_nm = araby.strip_tashkeel(enclitic)
        newsuffix = suffix  #default value
        #if the word ends by a haraka
        if not enclitic_nm and word[-1:] in (
                araby.ALEF_MAKSURA, araby.YEH,
                araby.ALEF) and araby.is_haraka(suffix):
            newsuffix = u""

        #gererate the suffix without I'rab short mark
        # here we lookup with given suffix because the new suffix is
        # changed and can be not found in table
        if u'متحرك' in ssconst.CONJ_SUFFIX_LIST_TAGS[suffix]['tags']:
            suffix_non_irab_mark = araby.strip_lastharaka(newsuffix)
        else:
            suffix_non_irab_mark = newsuffix

        return newsuffix, suffix_non_irab_mark

    @staticmethod
    def get_enclitic_variants(word, enclitic):
        """
        Get the enclitic variant to be joined to the word.
        For example: word = عن, suffix = , encletic = ني.
        The word and enclitic are geminated.
        @param word: word found in dictionary.
        @type word: unicode.
        @param enclitic: first level suffix.
        @type enclitic: unicode.
        @return: variant of suffixes  (vocalized suffix and vocalized
        suffix without I'rab short mark).
        @rtype: (unicode, unicode)
        """
        #enclitic_nm = araby.strip_tashkeel(enclitic)
        #newsuffix = suffix #default value
        #if the word ends by a haraka
        # الإدغام في النون والياء في مثل فيّ، إليّ، عنّا ، منّا
        if enclitic.startswith(araby.NOON) and word.endswith(araby.NOON):
            enclitic = enclitic[1:] + araby.SHADDA
            #~ print "xxxxxxxxxxx--1"
        if enclitic.startswith(araby.KASRA + araby.YEH) and word.endswith(
                araby.YEH):
            enclitic = enclitic[1:] + araby.SHADDA
            #~ print "xxxxxxxxxxx--2"

        return enclitic

    @staticmethod
    def get_word_variant(word, suffix):
        """
        Get the word variant to be joined to the suffix.
        For example: word = مدرسة, suffix = ي. The word is converted to مدرست.
        @param word: word found in dictionary.
        @type word: unicode.
        @param suffix: suffix ( firts or second level).
        @type suffix: unicode.
        @return: variant of word.
        @rtype: unicode.
        """
        word_stem = word
        suffix_nm = araby.strip_tashkeel(suffix)

        # تحويل الألف المقصورة إلى ياء في مثل إلى => إليك
        if word_stem.endswith(araby.ALEF_MAKSURA) and suffix_nm:
            if word_stem == u"سِوَى":
                word_stem = word_stem[:-1] + araby.ALEF
            else:
                word_stem = word_stem[:-1] + araby.YEH + araby.SUKUN
        # تحويل الهمزة حسب موقعها
        elif word_stem.endswith(araby.HAMZA) and suffix_nm:
            if suffix.startswith(araby.DAMMA):
                word_stem = word_stem[:-1] + araby.WAW_HAMZA
            elif suffix.startswith(araby.KASRA):
                word_stem = word_stem[:-1] + araby.YEH_HAMZA

        # this option is not used with stop words, because most of them are not inflected مبني
        #if the word ends by a haraka strip the haraka if the suffix is not null
        if suffix and suffix[0] in araby.HARAKAT:
            word_stem = araby.strip_lastharaka(word_stem)

        # الإدغام في النون والياء في مثل فيّ، إليّ، عنّا ، منّا
        if suffix.startswith(
                araby.NOON) and word.endswith(araby.NOON + araby.SUKUN):
            word_stem = araby.strip_lastharaka(word_stem)
        elif suffix.startswith(araby.KASRA + araby.YEH) and word.endswith(
                araby.YEH + araby.SUKUN):
            word_stem = araby.strip_lastharaka(word_stem)

        return word_stem

    def vocalize(self, stop, proclitic, suffix, enclitic):
        """
        Join the  stop and its affixes, and get the vocalized form
        @param stop: stop found in dictionary.
        @type stop: unicode.
        @param proclitic: first level prefix.
        @type proclitic: unicode.

        @param suffix: second level suffix.
        @type suffix: unicode.
        @param enclitic: first level suffix.
        @type enclitic: unicode.
        @return: vocalized word.
        @rtype: unicode.
        """
        # procletic have only an uniq vocalization in arabic
        proclitic_voc = ssconst.COMP_PREFIX_LIST_TAGS[proclitic]["vocalized"][0]
        # enclitic can have many vocalization in arabic
        # like heh => عليهِ سواهُ
        # in this stage we consider only one,
        # the second situation is ajusted by vocalize_ajust
        enclitic_voc = ssconst.COMP_SUFFIX_LIST_TAGS[enclitic]["vocalized"][0]
        suffix_voc = suffix  #CONJ_SUFFIX_LIST_TAGS[suffix]["vocalized"][0]

        # generate the word variant for some words witch ends by special
        #letters like Alef_maksura, or hamza,
        #the variant is influed by the suffix harakat,
        # for example إلي +ك = إلى+ك
        stop = self.get_word_variant(stop, suffix + enclitic)

        # generate the suffix variant. if the suffix is removed for some letters like
        # Alef Maqsura and Yeh
        # for example
        suffix_voc, suffix_non_irab_mark = self.get_suffix_variants(
            stop, suffix_voc, enclitic_voc)

        # generate the suffix variant. if the suffix is Yeh or Noon for geminating
        # for example عنّي = عن+ني
        enclitic_voc = self.get_enclitic_variants(stop, enclitic_voc)

        # generate the non vacalized end word: the vocalized word
        # without the I3rab Mark
        # if the suffix is a short haraka
        word_non_irab_mark = ''.join(
            [proclitic_voc, stop, suffix_non_irab_mark, enclitic_voc])

        word_vocalized = ''.join([proclitic_voc, stop, suffix_voc, enclitic_voc])
        return word_vocalized, word_non_irab_mark

    @staticmethod
    def verify_affix(word, list_seg, affix_list):
        """
        Verify possible affixes in the resulted segments according
        to the given affixes list.
        @param word: the input word.
        @type word: unicode.
        @param list_seg: list of word segments indexes (numbers).
        @type list_seg: list of pairs.
        @return: list of acceped segments.
        @rtype: list of pairs.
        """
        return [
            s for s in list_seg
            if '-'.join([word[:s[0]], word[s[1]:]]) in affix_list
        ]

    @staticmethod
    def validate_tags(stop_tuple, affix_tags, procletic, encletic_nm):
        """
        Test if the given word from dictionary is compabilbe with affixes tags.
        @param stop_tuple: the input word attributes given from dictionary.
        @type stop_tuple: dict.
        @param affix_tags: a list of tags given by affixes.
        @type affix_tags:list.
        @param procletic: first level prefix vocalized.
        @type procletic: unicode.
        @param encletic_nm: first level suffix vocalized.
        @type encletic_nm: unicode.
        @return: if the tags are compaatible.
        @rtype: Boolean.
        """
        procletic = araby.strip_tashkeel(procletic)
        #~ encletic = encletic_nm
        #~ suffix = suffix_nm

        if u"تعريف" in affix_tags and not stop_tuple['definition']:
            return False
        if u"تعريف" in affix_tags and stop_tuple['defined']:
            return False
        #~preposition
        if u'جر' in affix_tags and stop_tuple['is_inflected'] and not u"مجرور"  in affix_tags:
            return False
        if u'جر' in affix_tags and not stop_tuple['preposition']:
            return False
        if u"متحرك" in affix_tags and not stop_tuple['is_inflected']:
            return False

        if u"مضاف" in affix_tags and not stop_tuple['pronoun']:
            return False
        if u"مضاف" in affix_tags and stop_tuple['defined']:
            return False
        # حين تكون الأداة متحركة فهي تقبل الاتصال بياء المتكلم مباشرة
        if encletic_nm == araby.YEH and not stop_tuple['is_inflected']:
            return False
        # noon wiqaya نون الوقاية
        # حين تكون الأداة غير متحركة فهي تلزم  الاتصال بنون الوقاية قبل ياء المتكلم مباشرة
        if u"وقاية" in affix_tags and (stop_tuple['is_inflected']
                                       or stop_tuple['word'].endswith(araby.YEH)):
            return False
            #~interrog
        if u"استفهام" in affix_tags and not stop_tuple['interrog']:
            return False
            #~conjugation
            #~qasam

        if u"قسم" in affix_tags and not stop_tuple['qasam']:
            return False
            #~
            #~defined
            #~is_inflected
            #~tanwin
        if u"تنوين" in affix_tags and not stop_tuple['tanwin']:
            return False
            #~action
            #~object_type
            #~need
        return True

    @staticmethod
    def create_dict_word(dict_entries_list):
        """
        Create a list of dictWord objects from dictionary entries
        @param dict_entries_list: a list of entiers from lexicon
        @type  dict_entries_list: list of dict
        @return: a list of dictWord object
        @rtype: a list of dictWord object
        """
        return dict_entries_list

    @staticmethod
    def ajust_vocalization(vocalized):
        """
        ajust vocalization
        Temporary function
        @param vocalized: vocalized word.
        @type vocalized: unicode.
        @return: ajusted vocalized word.
        @rtype: unicode.
        """
        ajusted = ssconst.AJUSTMENT.get(vocalized, vocalized)

        return ajusted



#!/usr/bin/python
# -*- coding=utf-8 -*-
#-----------------------------------------------------------------------
# Name:        analex
# Purpose:     Arabic lexical analyser, provides feature to stem arabic
# words as noun, verb, stopword
#
# Author:      Taha Zerrouki (taha.zerrouki[at]gmail.com)
#
# Created:     31-10-2011
# Copyright:   (c) Taha Zerrouki 2011
# Licence:     GPL
#-----------------------------------------------------------------------
"""
    Arabic text morphological analyzer.
    Provides routins  to alanyze text.
    Can treat text as verbs or as nouns.
"""
from __future__ import (
    absolute_import,
    print_function,
    unicode_literals,
    #~ division,
    )
if __name__ == "__main__":
    import sys
    sys.path.append('..')
try:
    unicode
except NameError:
    from six import text_type as unicode
import re
import pyarabic.araby as araby 
from pyarabic.arabrepr import arepr
#~ import qalsadi.analex_const as analex_const  # special constant for analex
#~ import qalsadi.stem_noun as stem_noun  # noun stemming
#~ import qalsadi.stem_verb as stem_verb  # verb stemming
#~ import qalsadi.stem_unknown as stem_unknown  # unknown word stemming
#~ #~import qalsadi.stem_stopwords as  stem_stopwords     # stopwords word stemming
#~ import qalsadi.stem_stop as stem_stopwords  # stopwords word stemming
#~ import qalsadi.stem_pounct_const as stem_pounct_const  # pounctaution constants
#~ import naftawayh.wordtag  # word tagger
#~ import arramooz.wordfreqdictionaryclass as wordfreqdictionaryclass
#~ import qalsadi.disambig as disambig  # disambiguation const
#~ import qalsadi.wordcase as wordcase
#~ import qalsadi.stemmedword as stemmedword  # the result object for stemming
#~ import qalsadi.cache as cache

import arramooz.wordfreqdictionaryclass as wordfreqdictionaryclass
import naftawayh.wordtag  # word tagger
from . import analex_const  # special constant for analex
from . import stem_noun  # noun stemming
from . import stem_verb  # verb stemming
from . import stem_unknown  # unknown word stemming
from . import stem_stop as stem_stopwords  # stopwords word stemming
from . import stem_pounct_const  # pounctaution constants
from . import disambig  # disambiguation const
from . import wordcase
from . import stemmedword  # the result object for stemming
from . import cache
from . import cache_pickledb as cache

class Analex:
    """
        Arabic text morphological analyzer.
        Provides routins  to alanyze text.
        Can treat text as verbs or as nouns.
    """

    def __init__(self, cache_path=False ,allow_tag_guessing=True, allow_disambiguation=True):
        """
        Create Analex instance.
        """

        self.nounstemmer = stem_noun.NounStemmer()  # to stem nouns
        self.verbstemmer = stem_verb.VerbStemmer()  # to stem verbs
        self.unknownstemmer = stem_unknown.UnknownStemmer()
        # to stem unknown
        self.stopwordsstemmer = stem_stopwords.StopWordStemmer()
        # to stem stopwords

        self.allow_tag_guessing = allow_tag_guessing
        # allow gueesing tags by naftawayh before analyis
        # if taggin is disabled, the disambiguation is also disabled
        self.allow_disambiguation = allow_disambiguation and allow_tag_guessing
        # allow disambiguation before analyis
        # enable the last mark (Harakat Al-I3rab)
        self.allow_syntax_lastmark = True
        if self.allow_tag_guessing:
            self.tagger = naftawayh.wordtag.WordTagger()
        if self.allow_disambiguation:
            self.disambiguator = disambig.Disambiguator()
        self.debug = False  # to allow to print internal data
        self.limit = 10000  # limit words in the text
        self.wordcounter = 0
        # the words contain arabic letters and harakat.
        # the unicode considers arabic harakats as marks not letters,
        # then we add harakat to the regluar expression to tokenize
        marks = u"".join(araby.TASHKEEL)
        # contains [FATHA, DAMMA, KASRA, SUKUN, DAMMATAN, KASRATAN,
        #  FATHATAN, SHADDA])
        # used to tokenize arabic text
        #decprecated, we use araby.tokenize
        #~ self.token_pat = re.compile(ur"([\w%s]+)" % marks, re.UNICODE)
        #used to split text into clauses
        self.clause_pattern = re.compile(
            "([\w%s\s]+)" % (u"".join(araby.TASHKEEL), ), re.UNICODE)

        # allow partial vocalization support,
        #~The text is analyzed as partial or fully vocalized.
        self.partial_vocalization_support = True

        #word frequency dictionary
        self.wordfreq = wordfreqdictionaryclass.WordFreqDictionary(
            'wordfreq', wordfreqdictionaryclass.WORDFREQ_DICTIONARY_INDEX)

        # added to avoid duplicated search in the word frequency database
        # used as cache to reduce database access
        #added as a global variable to avoid duplucated search
        #in mutliple call of analex
        # cache used to avoid duplicata
        #~ self.allow_cache_use = True
        self.allow_cache_use = False
        #~ self.cache = cache.Cache(cache_path)
        self.cache_path = cache_path
        if cache_path:
            self.cache = cache.Cache(cache_path)
        else:
            self.cache = None        
        # In case of training and vocalized text analysis, 
        # we propose to respect Shadda in the given word
        self.fully_vocalized_input = False
        self.error_code = ""
        self.wordfreq_cache = {}

    def __del__(self):
        """
        Delete instance and clear cache
        """
        self.wordfreq = None
        self.nounstemmer = None
        self.verbstemmer = None
        self.unknownstemmer = None
        self.stopwordsstemmer = None
        self.tagger = None
        self.disambiguator = None
    def count_word(self, ):
        """ count input words. Used just for profiling and tests.
        @return: counter.
        @rtype: integer.
        """
        self.wordcounter += 1
        return self.wordcounter
    def get_freq(self, word, wordtype):
        """
        Return word_frequency
        """
        if word in self.wordfreq_cache:
            return self.wordfreq_cache[word]
        else:
            freq  = self.wordfreq.get_freq(word, wordtype)
            self.wordfreq_cache[word] = freq
            return  freq 

    def get_error_code(self,):
        """
        Return error code when word is not recognized
        """
        return "N%s-V%s"%(self.nounstemmer.get_error_code(),self.verbstemmer.get_error_code(), )
    def tokenize(self, text=u""):
        """
        Tokenize text into words
        @param text: the input text.
        @type text: unicode.
        @return: list of words.
        @rtype: list.
        """
        return araby.tokenize(text)

    def split_into_phrases(self, text):
        """
        Split Text into clauses
        @param text: input text
        @type text: unicode
        @return: list of clauses
        @rtype: list of unicode
        """
        if text:
            list_phrase = self.clause_pattern.split(text)
            if list_phrase:
                j = -1
                newlist = []
                for phr in list_phrase:
                    if not self.clause_pattern.match(phr):
                        #is pounctuation or symboles
                        #print 'not match', ph.encode('utf8')
                        if j < 0:
                            # the symbols are in the begining
                            newlist.append(phr)
                            j = 0
                        else:
                            # the symbols are after a phrases
                            newlist[j] += phr
                    else:
                        newlist.append(phr)
                        j += 1
                return newlist
            else:
                return []
        return []

    def text_tokenize(self, text):
        """
        Tokenize text into words, after treatement.
        @param text: the input text.
        @type text: unicode.
        @return: list of words.
        @rtype: list.
        """
        #~text = self.text_treat(text)
        list_word = self.tokenize(text)
        # avoid null values
        list_word = [word for word in list_word if word]
        #print "text_tokenize", u" ".join(list_word).encode('utf8')
        return list_word

    def set_debug(self, debug):
        """
        Set the debug attribute to allow printing internal analysis results.
        @param debug: the debug value.
        @type debug: True/False.
        """
        self.debug = debug
        self.nounstemmer.set_debug(debug)  # to set debug on noun stemming
        self.verbstemmer.set_debug(debug)  # to set debug on verb stemming

    def enable_syntax_lastmark(self):
        """
        Enable the syntaxic last mark attribute to allow use of I'rab harakat.
        """
        self.allow_syntax_lastmark = True
        self.nounstemmer.enable_syntax_lastmark()
        # to allow syntax last mark on noun stemming
        self.verbstemmer.enable_syntax_lastmark()
        # to allow syntax last mark  on verb stemming
        
    def enable_fully_vocalized_input(self):
        """
        Enable fully vocalized  input in case of vocalized text analysis.
        """
        self.fully_vocalized_input = True

    def disable_syntax_lastmark(self):
        """
        Disable the syntaxic last mark attribute to allow use of I'rab harakat.
        """
        self.allow_syntax_lastmark = False
        self.nounstemmer.disable_syntax_lastmark()
        # to allow syntax last mark on noun stemming
        self.verbstemmer.disable_syntax_lastmark()
        # to allow syntax last mark  on verb stemming

    def set_limit(self, limit):
        """
        Set the number of word treated in text.
        @param limit: the word number limit.
        @type limit: integer.
        """
        self.limit = limit

    def set_cacher(self, cacher=None):
        """
        Use a cache system to qalsadi to be used 
        """
        self.cache = cacher

    def enable_allow_cache_use(self):
        """
        Allow the analex to use Cache to reduce calcul.
        """
        self.allow_cache_use = True
        if not self.cache:
            self.cache = cache.Cache(self.cache_path)        

    def disable_allow_cache_use(self):
        """
        Not allow the analex to use Cache to reduce calcul.
        """
        self.allow_cache_use = False

    def check_text(self, text, mode='all'):
        """
        Analyze text morphologically.
        @param text: the input text.
        @type text: unicode.
        @param mode: the mode of analysis as 'verbs', 'nouns', or 'all'.
        @type mode: unicode.
        @return: list of dictionaries of analyzed words with tags.
        @rtype: list.
        """
        #print "ok", text.encode('utf8')
        list_word = self.text_tokenize(text)
        #print "ok", u"\t".join(list_word).encode('utf8')
        if self.allow_tag_guessing:
            list_guessed_tag = self.tagger.word_tagging(list_word)
            # avoid errors
            if len(list_guessed_tag) != len(list_word):
                #if the two lists have'nt the same length,
                # we construct a empty list for tags with the same length
                # print "error on guess tags"
                # sys.exit()
                list_guessed_tag = ['nv'] * len(list_word)
        # disambiguate  some words to speed up the analysis
        if self.allow_disambiguation:
            newwordlist = self.disambiguator.disambiguate_words(
                list_word, list_guessed_tag)
            # avoid the incomplete list
            if len(newwordlist) == len(list_word):
                list_word = newwordlist
                # print u" ".join(list_word).encode('utf8')
                # print u" ".join(list_guessed_tag).encode('utf8')

        #~ resulted_text = u""
        resulted_data = []

        #checkedWords = {} #global
        if mode == 'all':

            for i in list(range(len(list_word[:self.limit]))):
                word = list_word[i]
                self.count_word(
                )  # a ghost function to count words check function calls
                guessedtag = list_guessed_tag[i]
                one_data_list = self.check_word(word, guessedtag)
                stemmed_one_data_list = [
                    stemmedword.StemmedWord(w) for w in one_data_list
                ]
                resulted_data.append(stemmed_one_data_list)
        elif mode == 'nouns':

            for word in list_word[:self.limit]:
                one_data_list = self.check_word_as_noun(word)
                stemmed_one_data_list = [stemmedword.StemmedWord(w) \
                for w in one_data_list]
                resulted_data.append(stemmed_one_data_list)
                #~ resulted_data.append(one_data_list)
        elif mode == 'verbs':
            for word in list_word[:self.limit]:
                one_data_list = self.check_word_as_verb(word)
                stemmed_one_data_list = [stemmedword.StemmedWord(w) \
                for w in one_data_list]
                resulted_data.append(stemmed_one_data_list)
                #~ resulted_data.append(one_data_list)
        return resulted_data

    def check_word(self, word, guessedtag=""):
        """
        Analyze one word morphologically as verbs
        @param word: the input word.
        @type word: unicode.
        @return: list of dictionaries of analyzed words with tags.
        @rtype: list.
        """

        word = araby.strip_tatweel(word)
        word_vocalised = word
        word_nm = araby.strip_tashkeel(word)
        # get analysed details from cache if used
        if self.allow_cache_use and self.cache.is_already_checked(word_nm):
            #~ print (u"'%s'"%word).encode('utf8'), 'found'
            resulted_data = self.cache.get_checked(word_nm)
        else:
            resulted_data = []
            # if word is a pounctuation
            resulted_data += self.check_word_as_pounct(word_nm)
            # Done: if the word is a stop word we have  some problems,
            # the stop word can also be another normal word (verb or noun),
            # we must consider it in future works
            # if word is stopword allow stop words analysis
            if araby.is_arabicword(word_nm):
                resulted_data += self.check_word_as_stopword(word_nm)

                #if word is verb
                # مشكلة بعض الكلمات المستبعدة تعتبر أفعلا أو اسماء
                #~if  self.tagger.has_verb_tag(guessedtag) or \
                #~self.tagger.is_stopword_tag(guessedtag):
                #~resulted_data += self.check_word_as_verb(word_nm)
                resulted_data += self.check_word_as_verb(word_nm)
                #print "is verb", rabti, len(resulted_data)
                #if word is noun
                #~if self.tagger.has_noun_tag(guessedtag) or \
                #~self.tagger.is_stopword_tag(guessedtag):
                #~resulted_data += self.check_word_as_noun(word_nm)
                resulted_data += self.check_word_as_noun(word_nm)
            if len(resulted_data) == 0:
                #print (u"1 _unknown %s-%s"%(word, word_nm)).encode('utf8')
                #check the word as unkonwn
                resulted_data += self.check_word_as_unknown(word_nm)
                #check if the word is nomralized and solution are equivalent
            resulted_data = self.check_normalized(word_vocalised, resulted_data)
            #check if the word is shadda like
            
            resulted_data = self.check_shadda(word_vocalised, resulted_data, self.fully_vocalized_input)

            # add word frequency information in tags
            resulted_data = self.add_word_frequency(resulted_data)

            # add the stemmed words details into Cache
            data_list_to_serialize = [w.__dict__ for w in resulted_data]
            if self.allow_cache_use:
                self.cache.add_checked(word_nm, data_list_to_serialize)

        #check if the word is vocalized like results
        if self.partial_vocalization_support:
            resulted_data = self.check_partial_vocalized(word_vocalised,
                                                    resulted_data)

        if len(resulted_data) == 0:
            error_code = self.get_error_code()
            resulted_data.append(
                wordcase.WordCase({
                    'word':
                    word,
                    'affix': ('', '', '', ''),
                    'stem':
                    word,
                    'original':
                    word,
                    'vocalized':
                    word,
                    'semivocalized':
                    word,
                    'tags':
                    u'%s'%error_code,
                    'type':
                    'unknown',
                    'root':
                    '',
                    'template':
                    '',
                    'freq':
                    self.get_freq(word, 'unknown'),
                    'syntax':
                    '',
                }))
        return resulted_data

    def check_text_as_nouns(self, text):
        """
        Analyze text morphologically as nouns
        @param text: the input text.
        @type text: unicode.
        @return: list of dictionaries of analyzed words with tags.
        @rtype: list.
        """
        return self.check_text(text, "nouns")

    def check_text_as_verbs(self, text):
        """
        Analyze text morphologically as verbs
        @param text: the input text.
        @type text: unicode.
        @return: list of dictionaries of analyzed words with tags.
        @rtype: list.
        """
        return self.check_text(text, "verbs")

    def add_word_frequency(self, resulted_data):
        """
        If the entred word is like the found word in dictionary,
        to treat some normalized cases,
        the analyzer return the vocalized like words
        ُIf the word is ذئب, the normalized form is ذءب, which can give
        from dictionary ذئبـ ذؤب.
        this function filter normalized resulted word according the
        given word, and give ذئب.
        @param resulted_data: the founded resulat from dictionary.
        @type resulted_data: list of dict.
        @return: list of dictionaries of analyzed words with tags.
        @rtype: list.
        """
        # added to avoid duplicated search in the word frequency database
        # used as cache to reduce database access
        #added as a global variable to avoid duplucated search
        #in mutliple call of analex
        #checkedFreqWords = {'noun':{}, 'verb':{}} # global
        for i, item in enumerate(resulted_data):
            # get the original word of dictionary,
            #~ item = resulted_data[i]
            # search for the original (lexique entry)
            #~ original = item.get('original', '')
            #~ original = item.__dict__.get('original', '')
            original = item.original
            # in the freq attribute we found 'freqverb, or freqnoun,
            #  or a frequency for stopwords or unkown
            # the freqtype is used to note the wordtype,
            # this type is passed by stem_noun , or stem_verb modules
            freqtype = item.freq
            #~ freqtype = item.get('freq', '')
            if freqtype == 'freqverb':
                wordtype = 'verb'
            elif freqtype == 'freqnoun':
                wordtype = 'noun'
            elif freqtype == 'freqstopword':
                wordtype = 'stopword'
            else:
                wordtype = ''
            if wordtype:
                # if frequency is already get from database,
                #  don't access to database
                if self.allow_cache_use and self.cache.exists_cache_freq(
                        original, wordtype):
                    item.freq = self.cache.get_freq(original, wordtype)

                else:
                    freq = self.get_freq(original, wordtype)
                    #~print freq, wordtype, original.encode('utf8')
                    #store the freq in the cache
                    if self.allow_cache_use:
                        #~ self.cache['FreqWords'][wordtype][original] = freq
                        self.cache.add_freq(original, wordtype, freq)

                    #~ item.__dict__['freq'] = freq
                    #~ item.__dict__['freq'] = freq
                    item.freq = freq
            resulted_data[i] = item
        return resulted_data

    def check_word_as_stopword(self, word):
        """
        Check if the word is a stopword,
        @param word: the input word.
        @type word: unicode.
        @return: list of dictionaries of analyzed words with tags.
        @rtype: list.
        """
        return self.stopwordsstemmer.stemming_stopword(word)

    def check_word_as_pounct(self, word):
        """
        Check if the word is a pounctuation,
        @param word: the input word.
        @type word: unicode.
        @return: list of dictionaries of analyzed words with tags.
        @rtype: list.
        """
        detailed_result = []
        if not word:
            return detailed_result
        # ToDo : fix it to isdigit, by moatz saad
        if word.isnumeric():
            detailed_result.append(
                wordcase.WordCase({
                    'word': word,
                    'affix': ('', '', '', ''),
                    'stem': '',
                    'original': word,
                    'vocalized': word,
                    'tags': u"عدد",
                    'type': 'NUMBER',
                    'freq': 0,
                    'syntax': '',
                    'root':'',
                }))
        # test if all chars in word are punctuation
        for char in word:
            # if one char is not a pounct, break
            if char not in stem_pounct_const.POUNCTUATION:
                break
        else:
            # if all chars are pounct, the word take tags of the first char
            detailed_result.append(
                wordcase.WordCase({
                    'word':
                    word,
                    'affix': ('', '', '', ''),
                    'stem':
                    '',
                    'original':
                    word,
                    'vocalized':
                    word,
                    'tags':
                    stem_pounct_const.POUNCTUATION[word[0]]['tags'],
                    'type':
                    'POUNCT',
                    'freq':
                    0,
                    'syntax':
                    '',
                    'root':'',
                }))

        return detailed_result

    def check_word_as_verb(self, verb):
        """
        Analyze the word as verb.
        @param verb: the input word.
        @type verb: unicode.
        @return: list of dictionaries of analyzed words with tags.
        @rtype: list.
        """
        return self.verbstemmer.stemming_verb(verb)

    def check_word_as_noun(self, noun):
        """
        Analyze the word as noun.
        @param noun: the input word.
        @type noun: unicode.
        @return: list of dictionaries of analyzed words with tags.
        @rtype: list.
        """
        return self.nounstemmer.stemming_noun(noun)

    def check_word_as_unknown(self, noun):
        """
        Analyze the word as unknown.
        @param noun: the input word.
        @type noun: unicode.
        @return: list of dictionaries of analyzed words with tags.
        @rtype: list.
        """
        return self.unknownstemmer.stemming_noun(noun)

    @staticmethod
    def check_shadda(word_vocalised, resulted_data, fully_vocalized_input=False):
        """
        if the entred word is like the found word in dictionary,
        to treat some normalized cases,
        the analyzer return the vocalized like words.
        This function treat the Shadda case.
        @param word_vocalised: the input word.
        @type word_vocalised: unicode.
        @param resulted_data: the founded resulat from dictionary.
        @type resulted_data: list of dict.
        @param fully_vocalized_input: if the two words must resect the shadda and vocalized.
        @type fully_vocalized_input: Boolean, default is False.
        @return: list of dictionaries of analyzed words with tags.
        @rtype: list.
        """
        #~return filter(lambda item: araby.shaddalike(word_vocalised,
        #~item.__dict__.get('vocalized', '')), resulted_data)
        #~x for x in [1, 1, 2] if x == 1
        #~ return [
        #~ x for x in resulted_data
        #~ if araby.shaddalike(word_vocalised, x.__dict__.get('vocalized', '')) ]
        if fully_vocalized_input:
            return [x for x in resulted_data if araby.strip_harakat(word_vocalised) == 
        #~ araby.strip_harakat(x.__dict__.get('vocalized', ''))]
        araby.strip_harakat(x.vocalized)]
        else:
            return [
            x for x in resulted_data
            #~ if araby.shaddalike(word_vocalised, x.__dict__.get('vocalized', ''))
            if araby.shaddalike(word_vocalised, x.vocalized)
        ]
        

    #~ @staticmethod
    def check_normalized(self, word_vocalised, resulted_data):
        """
        If the entred word is like the found word in dictionary,
        to treat some normalized cases,
        the analyzer return the vocalized like words
        ُIf the word is ذئب, the normalized form is ذءب,
        which can give from dictionary ذئبـ ذؤب.
        this function filter normalized resulted word according
        the given word, and give ذئب.
        @param word_vocalised: the input word.
        @type word_vocalised: unicode.
        @param resulted_data: the founded resulat from dictionary.
        @type resulted_data: list of dict.
        @return: list of dictionaries of analyzed words with tags.
        @rtype: list.
        """
        #print word_vocalised.encode('utf8')
        filtred_data = []
        inputword = araby.strip_tashkeel(word_vocalised)
        for item in resulted_data:
            #~ if 'vocalized' in item.__dict__: 
            if hasattr(item, 'vocalized'): 
                #~ if 'vocalized' in item :
                outputword = araby.strip_tashkeel(item.vocalized)
                if self.debug:
                    print(u'\t'.join([inputword, outputword]).encode('utf8'))
                if inputword == outputword:
                    #item['tags'] += ':a'
                    filtred_data.append(item)
                #~ filtred_data.append(item)
        return filtred_data

    @staticmethod
    def check_partial_vocalized(word_vocalised, resulted_data):
        """
        if the entred word is vocalized fully or partially,
        the analyzer return the vocalized like words
        This function treat the partial vocalized case.
        @param word_vocalised: the input word.
        @type word_vocalised: unicode.
        @param resulted_data: the founded resulat from dictionary.
        @type resulted_data: list of dict.
        @return: list of dictionaries of analyzed words with tags.
        @rtype: list.
        """
        filtred_data = []
        if not araby.is_vocalized(word_vocalised):
            return resulted_data
        else:
            #compare the vocalized output with the vocalized input
            #print ' is vocalized'
            for item in resulted_data:
                if 'vocalized' in item:
                    output = item['vocalized']
                    is_verb = "Verb" in item['type']
                    if araby.vocalizedlike(word_vocalised, output):
                        item['tags'] += ':' + analex_const.PARTIAL_VOCALIZED_TAG
                        filtred_data.append(item)
                        # حالة التقا الساكنين، مع نص مشكول مسبقا، والفعل في آخره كسرة بدل السكون
                    elif is_verb and  word_vocalised.endswith(araby.KASRA) and output.endswith(araby.SUKUN):
                        if araby.vocalizedlike(word_vocalised[:-1], output[:-1]):
                            item['tags'] += ':' + analex_const.PARTIAL_VOCALIZED_TAG
                            filtred_data.append(item)
                    
        return filtred_data



#!/usr/bin/python
# -*- coding=utf-8 -*-
#-------------------------------------------------------------------------
# Name:        stem_noun
# Purpose:     Arabic lexical analyser, provides feature for
#~stemming arabic word as noun
#
# Author:      Taha Zerrouki (taha.zerrouki[at]gmail.com)
#
# Created:     31-10-2011
# Copyright:   (c) Taha Zerrouki 2011
# Licence:     GPL
#-------------------------------------------------------------------------
"""
    Arabic noun stemmer
"""
#~ from __future__ import (
    #~ absolute_import,
    #~ print_function,
    #~ unicode_literals,
    #~ )
import re
if __name__ == '__main__':
    import sys
    sys.path.append('../support')
    sys.path.append('support')
    sys.path.append('..')
import pyarabic.araby as ar
from pyarabic.arabrepr import arepr
import tashaphyne.stemming
#~ import tashaphyne.normalize
import alyahmor.aly_stem_noun_const as SNC
import alyahmor.noun_affixer
import arramooz.arabicdictionary as arabicdictionary
from .print_debug import print_table
from . import custom_dictionary
from . import wordcase

class NounStemmer:
    """
        Arabic noun stemmer
    """

    def __init__(self, debug=False):
        # create a stemmer object for stemming enclitics and proclitics
        self.comp_stemmer = tashaphyne.stemming.ArabicLightStemmer()
        # configure the stemmer object
        self.comp_stemmer.set_prefix_list(SNC.COMP_PREFIX_LIST)
        self.comp_stemmer.set_suffix_list(SNC.COMP_SUFFIX_LIST)
        # create a stemmer object for stemming conjugated verb
        self.conj_stemmer = tashaphyne.stemming.ArabicLightStemmer()
        # configure the stemmer object
        self.conj_stemmer.set_prefix_list(SNC.CONJ_PREFIX_LIST)
        self.conj_stemmer.set_suffix_list(SNC.CONJ_SUFFIX_LIST)
        
        # generator 
        self.generator = alyahmor.noun_affixer.noun_affixer()
        # enable the last mark (Harakat Al-I3rab)
        self.allow_syntax_lastmark = True

        # noun dictionary
        self.noun_dictionary = arabicdictionary.ArabicDictionary("nouns")
        # costum noun dictionary
        self.custom_noun_dictionary = custom_dictionary.custom_dictionary("nouns")

        # allow to print internal results.
        self.cache_dict_search = {}
        self.cache_affixes_verification = {}
        self.noun_cache = {}
        self.debug = debug
        self.error_code = ""
        
    def get_error_code(self,):
        """
        Return error code when word is not recognized
        """
        return self.error_code
    def set_error_code(self, error_code):
        """
        set error code when word is not recognized
        """
        if not self.error_code:
            self.error_code = error_code
    def lookup_dict(self, word):
        """
        lookup for word in dict
        """
        if word in self.noun_cache:
            return self.noun_cache[word]
        else:
            result = self.noun_dictionary.lookup(word)
            result +=  self.custom_noun_dictionary.lookup(word)
            self.noun_cache[word] = result
        return result
        
    def stemming_noun(self, noun_in):
        """
        Analyze word morphologically as noun
        @param noun_in: the input noun.
        @type noun_in: unicode.
        @return: list of dictionaries of analyzed words with tags.
        @rtype: list.
        """
        self.set_error_code('')
        if not noun_in:
            self.set_error_code('Empty word')
            return None
        debug = self.debug
        #~list_found = []
        detailed_result = []
        noun_list = [
            noun_in,
        ] + self.get_noun_variants(noun_in)
        word_segmented_list = []
        for noun in noun_list:
            list_seg_comp = self.comp_stemmer.segment(noun)
            # filter
            list_seg_comp = self.verify_affix(noun, list_seg_comp,
                                         SNC.COMP_NOUN_AFFIXES)
            # treat multi vocalization enclitic
            for seg in list_seg_comp:
                proclitic_nm = noun[:seg[0]]
                stem = noun[seg[0]:seg[1]]
                enclitic_nm = noun[seg[1]:]
                # ajusting nouns variant
                list_stem = [
                    stem,
                ] + self.get_input_stem_variants(stem, enclitic_nm)

                # stem reduced noun : level two
                for stem in list_stem:
                    word_seg = {
                        'noun': noun,
                        'stem_comp': stem,
                        'pro': proclitic_nm,
                        'enc': enclitic_nm,
                    }
                    word_segmented_list.append(word_seg)
        if not word_segmented_list:
            self.set_error_code(" First level segmentation error")

        # level two

        tmp_list = []
        if debug: print("after first level")
        if debug:
            #~ print(repr(word_segmented_list).replace(
                #~ '},', '},\n').decode("unicode-escape"))
            print(arepr(noun_in))
            print(print_table(word_segmented_list))

        for word_seg in word_segmented_list:

            #~ detailed_result.extend(
            #~ self.steming_second_level(word_seg['noun'], word_seg['stem_comp'],
            #~ word_seg['pro'], word_seg['enc']))
            #~ detailed_result_one = []
            #segment the coinjugated noun
            list_seg_conj = self.conj_stemmer.segment(word_seg['stem_comp'])
            # verify affix compatibility
            # filter
            list_seg_conj = self.verify_affix(word_seg['stem_comp'], list_seg_conj,
                                         SNC.NOMINAL_CONJUGATION_AFFIX)
            # add vocalized forms of suffixes
            # and create the real affixes from the word
            for seg_conj in list_seg_conj:
                stem_conj = word_seg['stem_comp'][:seg_conj[1]]
                suffix = word_seg['stem_comp'][seg_conj[1]:]
                stem_conj = ar.normalize_hamza(stem_conj)
                stem_conj_list = self.get_stem_variants(stem_conj, suffix)

                # generate possible stems
                # add stripped letters to the stem to constitute possible noun list
                for stem in stem_conj_list:
                    word_seg_l2 = word_seg.copy()
                    # normalize hamza before gessing  differents origines
                    word_seg_l2['stem_conj'] = stem
                    word_seg_l2['suffix'] = suffix
                    #affixes tags contains prefixes and suffixes tags
                    word_seg_l2['affix_tags'] = list(
                        set(SNC.COMP_PREFIX_LIST_TAGS[word_seg_l2['pro']]
                            ['tags'] + SNC.
                            COMP_SUFFIX_LIST_TAGS[word_seg_l2['enc']]['tags'] +
                            SNC.CONJ_SUFFIX_LIST_TAGS[word_seg_l2['suffix']]
                            ['tags']))
                    tmp_list.append(word_seg_l2)

        if debug: print("after second level")
        if debug:
            print(arepr(noun_in))
            print(print_table(tmp_list))
        # lookup in dictionary
        if not tmp_list:
            self.set_error_code(" Second level segmentation error")
        word_segmented_list = tmp_list
        tmp_list = []
        for word_seg in word_segmented_list:
            # search the noun in the dictionary
            # we can return the tashkeel
            inf_noun = word_seg['stem_conj']
            # get the noun and get all its forms from the dict
            # if the noun has plural suffix, don't look up in
            #broken plural dictionary
            if inf_noun in self.cache_dict_search:
                infnoun_foundlist = self.cache_dict_search[inf_noun]
            else:
                infnoun_foundlist = self.lookup_dict(inf_noun)
                self.cache_dict_search[inf_noun] = infnoun_foundlist

            for noun_tuple in infnoun_foundlist:
                word_seg_l3 = word_seg.copy()
                word_seg_l3["original"] = noun_tuple['vocalized']
                word_seg_l3["noun_tuple"] = dict(noun_tuple)
                tmp_list.append(word_seg_l3)

        if debug: print("after lookup dict")
        if debug:
            print(arepr(noun_in))
            noun_tuples = [item['noun_tuple'] for item in  tmp_list]        
            print(print_table(noun_tuples))
        # test compatiblity noun_tuple with affixes and proaffixes
        # and generate vocalized affixes and suffixes
        if not tmp_list:
            self.set_error_code("Not exists in dictionary")
        word_segmented_list = tmp_list
        tmp_list = []
        for word_seg in word_segmented_list:
            #test if the  given word from dictionary accept those
            # tags given by affixes
            # دراسة توافق الزوائد مع خصائص الاسم،
            # مثلا هل يقبل الاسم التأنيث.
            if self.validate_tags(word_seg['noun_tuple'], word_seg['affix_tags'],
                             word_seg['pro'], word_seg['enc'],
                             word_seg['suffix']):
                ## get all vocalized form of suffixes
                for pro_voc in SNC.COMP_PREFIX_LIST_TAGS[word_seg['pro']][
                        'vocalized']:
                    for enc_voc in SNC.COMP_SUFFIX_LIST_TAGS[word_seg['enc']][
                            'vocalized']:
                        for suf_voc in SNC.CONJ_SUFFIX_LIST_TAGS[word_seg[
                                'suffix']]['vocalized']:
                            ## verify compatibility between proclitics and affix
                            if self.__check_clitic_affix(word_seg['noun_tuple'],
                                                         pro_voc, enc_voc,
                                                         suf_voc):
                                # get affix tags
                                affix_tags_voc = SNC.COMP_PREFIX_LIST_TAGS[pro_voc]['tags']\
                                  +SNC.COMP_SUFFIX_LIST_TAGS[enc_voc]['tags']\
                                  +SNC.CONJ_SUFFIX_LIST_TAGS[suf_voc]['tags']
                                word_seg_l4 = word_seg.copy()
                                word_seg_l4['suf_voc'] = suf_voc
                                word_seg_l4['enc_voc'] = enc_voc
                                word_seg_l4['affix_tags'] = affix_tags_voc
                                tmp_list.append(word_seg_l4)

        if debug: print("after check compatibility")
        if debug:
            print(arepr(noun_in))
            noun_tuples = [item['noun_tuple'] for item in  tmp_list]        
            print(print_table(noun_tuples))
        # Generate results
        if not tmp_list:
            self.set_error_code("Affixes not compatible")

        word_segmented_list = tmp_list
        tmp_list = []
        for word_seg in word_segmented_list:
            # get voalized and vocalized without inflection
            #~ vocalized, semi_vocalized, _ = self.vocalize(
            voca_tuple_list = self.vocalize(
                word_seg['noun_tuple']['vocalized'], word_seg['pro'], word_seg['suf_voc'],
                word_seg['enc_voc'])
            for vocalized, semi_vocalized, _ in voca_tuple_list:
                #add some tags from dictionary entry as
                #mamnou3 min sarf and broken plural
                original_tags = []
                if word_seg['noun_tuple']['mankous'] == u"Tk":
                    original_tags.append(u"منقوص")
                # if there are many cases like feminin plural with mansoub and majrour
                if 'cases' in SNC.CONJ_SUFFIX_LIST_TAGS[word_seg['suf_voc']]:
                    list_cases = SNC.CONJ_SUFFIX_LIST_TAGS[word_seg['suf_voc']][
                        'cases']
                else:
                    list_cases = ('', )
                for case in list_cases:
                    voc_affix_case = word_seg['affix_tags'] + (case, )
                    # filter empty
                    voc_affix_case = [ vac for vac in voc_affix_case if vac]
                    detailed_result.append(
                        wordcase.WordCase({
                            'word':
                            noun_in,
                            'affix': (word_seg['pro'], '', word_seg['suf_voc'],
                                      word_seg['enc_voc']),
                            'stem':
                            word_seg['stem_conj'],
                            'root':ar.normalize_hamza(word_seg['noun_tuple'].get('root','')),
                            'original':
                            word_seg['noun_tuple']['vocalized'],  #original,
                            'vocalized':
                            vocalized,
                            'semivocalized':
                            semi_vocalized,
                            'tags':
                            u':'.join(voc_affix_case),
                            'type':
                            u':'.join(['Noun', word_seg['noun_tuple']['wordtype']]),
                            'number':
                            word_seg['noun_tuple']['number'],
                            'gender':
                            word_seg['noun_tuple']['gender'],
                            'freq':
                            'freqnoun',  # to note the frequency type
                            'originaltags':
                            u':'.join(original_tags),
                            'syntax':
                            '',
                        }))
        if not detailed_result:
            self.set_error_code("Forms are not generated")

        if debug: print("after generate result")
        if debug: print(len(detailed_result))
        #~ if debug: print repr(detailed_result).replace('},','},\n').decode("unicode-escape")
        return detailed_result

    def __check_clitic_affix(self, noun_tuple, proclitic_nm, enclitic, suffix):
        """
        Verify if proaffixes (sytaxic affixes) are compatable
        with affixes ( conjugation)
        @param proclitic_nm: first level prefix.
        @type proclitic_nm: unicode.
        @param enclitic: first level suffix.
        @type enclitic: unicode.
        @param suffix: second level suffix.
        @type suffix: unicode.
        @return: compatible.
        @rtype: True/False.
        """
        #~ print "XXXstem_noun", noun_tuple["unvocalized"].encode('utf8'), noun_tuple['mamnou3_sarf'],type(noun_tuple['mamnou3_sarf']),    bool(noun_tuple['mamnou3_sarf'])
        
        # avoid Fathatan on no ALEF Tawnwin expect on Teh marbuta and Alef followed by Hamza
        # تجنب تنوين النصب على غير الألف ما عدا التاء المربوطة أو همزة بعد ألف
        if suffix == ar.FATHATAN and not (
                noun_tuple["unvocalized"].endswith(ar.TEH_MARBUTA)
                or noun_tuple["unvocalized"].endswith(ar.ALEF + ar.HAMZA)):
            return False
        # avoid masculin regular plural with unallowed case
        # تجنب جمع المذكر السالم للكلمات التي لا تقبلها
        if u'جمع مذكر سالم' in SNC.CONJ_SUFFIX_LIST_TAGS[suffix]['tags']\
          and not noun_tuple['masculin_plural']:
            return False
                    # التنوين لا يتطابق مع الممنوع من الصرف
        #~ print "XXXstem_noun", noun_tuple["unvocalized"].encode('utf8'), noun_tuple['mamnou3_sarf'],type(noun_tuple['mamnou3_sarf']),    bool(noun_tuple['mamnou3_sarf'])
        #~ print "stem_noun", noun_tuple["unvocalized"].encode('utf8'), noun_tuple['masculin_plural'],type(noun_tuple['masculin_plural']),    bool(noun_tuple['masculin_plural'])
        if u'تنوين' in SNC.CONJ_SUFFIX_LIST_TAGS[suffix]['tags'] and noun_tuple['mamnou3_sarf']:
            return False
        #if not proclitic and not enclitic:  return True
        #use cache for affix verification
        affix = u'-'.join([
            proclitic_nm, enclitic, suffix,
            str(bool(noun_tuple['mamnou3_sarf']))
        ])
        #~print affix.encode("utf8")
        if affix in self.cache_affixes_verification:
            return self.cache_affixes_verification[affix]

        # get proclitics and enclitics tags
        proclitic_tags = SNC.COMP_PREFIX_LIST_TAGS[proclitic_nm]['tags']
        enclitic_tags = SNC.COMP_SUFFIX_LIST_TAGS[enclitic]['tags']
        # in nouns there is no prefix
        suffix_tags = SNC.CONJ_SUFFIX_LIST_TAGS[suffix]['tags']
        # in some cases the suffixes have more cases
        # add this cases to suffix tags
        suffix_tags += SNC.CONJ_SUFFIX_LIST_TAGS[suffix].get("cases", ())
        if u"تعريف" in proclitic_tags and u"مضاف" in suffix_tags and \
        u'مضاف' not in enclitic_tags:
            self.cache_affixes_verification[affix] = False
        elif u"تعريف" in proclitic_tags and u"تنوين" in suffix_tags:
            self.cache_affixes_verification[affix] = False
        elif u"تعريف" in proclitic_tags and u"إضافة" in suffix_tags:
            self.cache_affixes_verification[affix] = False


    # الجر  في حالات الاسم المعرفة بال أو الإضافة إلى ضمير أو مضاف إليه
    # مما يعني لا يمكن تطبيقها هنا
    # بل في حالة التحليل النحوي
        elif u"مضاف" in enclitic_tags and u"تنوين" in suffix_tags:
            self.cache_affixes_verification[affix] = False
        elif u"مضاف" in enclitic_tags and u"لايضاف" in suffix_tags:
            self.cache_affixes_verification[affix] = False
        elif u"جر" in proclitic_tags and u"مجرور" not in suffix_tags:
            self.cache_affixes_verification[affix] = False


#ستعمل في حالة كسر هاء الضمير في الجر

#elif  bool(u"لايجر" in enclitic_tags) and  bool(u"مجرور" in \
#suffix_tags) :
#    self.cache_affixes_verification[affix] = False
#elif  bool(u"مجرور" in enclitic_tags) and  not bool(u"مجرور" in \
#suffix_tags) :
#    self.cache_affixes_verification[affix] = False
        else:
            self.cache_affixes_verification[affix] = True

        return self.cache_affixes_verification[affix]

    def set_debug(self, debug):
        """
        Set the debug attribute to allow printing internal analysis results.
        @param debug: the debug value.
        @type debug: True/False.
        """
        self.debug = debug

    def enable_syntax_lastmark(self):
        """
        Enable the syntaxic last mark attribute to allow use of I'rab harakat.
        """
        self.allow_syntax_lastmark = True

    def disable_syntax_lastmark(self):
        """
        Disable the syntaxic last mark attribute to allow use of I'rab harakat.
        """
        self.allow_syntax_lastmark = False

    @staticmethod
    def get_stem_variants(stem, suffix_nm):
        """
        Generate the Noun stem variants according to the affixes.
        For example مدرستي = >مدرست+ي = > مدرسة +ي.
        Return a list of possible cases.
        @param stem: the input stem.
        @type stem: unicode.
        @param suffix_nm: suffix (no mark).
        @type suffix_nm: unicode.
        @return: list of stem variants.
        @rtype: list of unicode.
        """
        #some cases must have some correction
        #determinate the  suffix types
        #~suffix = suffix_nm

        possible_noun_list = set([
            stem,
        ])
        if suffix_nm in (ar.ALEF + ar.TEH, ar.YEH + ar.TEH_MARBUTA, ar.YEH,
                         ar.YEH + ar.ALEF + ar.TEH):
            possible_noun = stem + ar.TEH_MARBUTA
            possible_noun_list.add(possible_noun)
        # فئت +ان
        if suffix_nm in (ar.ALEF+ar.NOON, ar.YEH + ar.NOON) and stem.endswith(ar.TEH):
            possible_noun = stem[:-1] + ar.TEH_MARBUTA
            possible_noun_list.add(possible_noun)
            
            
        if not suffix_nm or suffix_nm in (ar.YEH + ar.NOON, ar.WAW + ar.NOON):
            possible_noun = stem + ar.YEH
            possible_noun_list.add(possible_noun)
        if stem.endswith(ar.YEH):
            # إذا كان أصل الياء ألفا مقصورة
            possible_noun = stem[:-1] + ar.ALEF_MAKSURA
            possible_noun_list.add(possible_noun)

        if stem.endswith(ar.HAMZA):
            possible_noun = stem[:-1] + ar.YEH_HAMZA
            possible_noun_list.add(possible_noun)
            #~ possible_noun = stem[:-1] + ar.WAW_HAMZA
            #~ possible_noun_list.add(possible_noun)
        #to be validated
        validated_list = possible_noun_list
        return validated_list

    @staticmethod
    def get_suffix_variants(word, suffix, enclitic, mankous=False):
        """
        Get the suffix variant to be joined to the word.
        For example: word = مدرس, suffix = ة, enclitic = ي.
        The suffix is converted to Teh.
        @param word: word found in dictionary.
        @type word: unicode.
        @param suffix: second level suffix.
        @type suffix: unicode.
        @param enclitic: first level suffix.
        @type enclitic: unicode.
        @param mankous: if the noun is mankous ends with Yeh منقوص.
        @type mankous: boolean.
        @return: variant of suffixes  (vocalized suffix and vocalized
        suffix without I'rab short mark).
        @rtype: (unicode, unicode)
        """
        #enclitic_nm = ar.strip_tashkeel(enclitic)
        enclitic_nm = enclitic  # given enclitic is not vocalized
        newsuffix = suffix  #default value
        #if the word ends by a haraka
        if suffix.find(ar.TEH_MARBUTA) >= 0 and enclitic_nm:
            newsuffix = re.sub(ar.TEH_MARBUTA, ar.TEH, suffix)

        elif not enclitic_nm and ar.is_haraka(suffix):
            if word[-1:] in (ar.YEH, ar.ALEF):
                newsuffix = u""
            elif mankous:
                # the word is striped from YEH المنقوص حذفت ياؤه قبل قليل
                # تحول حركته إلى تنوين كسر
                newsuffix = ar.KASRATAN
        #gererate the suffix without I'rab short mark
        # here we lookup with given suffix because the new suffix is
        # changed and can be not found in table
        if u'متحرك' in SNC.CONJ_SUFFIX_LIST_TAGS[suffix]['tags']:
            suffix_non_irab_mark = ar.strip_lastharaka(newsuffix)
        else:
            suffix_non_irab_mark = newsuffix
        return newsuffix, suffix_non_irab_mark

    @staticmethod
    def get_enclitic_variant(enclitic_voc, suffix_voc):
        """
        Get the enclitix variant to be joined to the word.
        For example: word = كتاب, suffix = كسرة, enclitic = هم.
        The enclitic has a second form هِم.
        @param enclitic_voc: first level suffix vocalized.
        @type enclitic_voc: unicode.
        @param suffix_voc: second level suffix vocalized.
        @type suffix_voc: unicode.
        @return: variant of enclitic  (vocalized enclitic and vocalized
        enclitic without I'rab short mark).
        @rtype: (unicode, unicode)
        """
        #print (u"get enclit2 '%s' %d"%(enclitic_voc, len(enclitic_voc))).encode('utf8')
        encl_vo_no_inflect_mark = enclitic_voc
        if enclitic_voc.startswith(ar.HEH + ar.DAMMA) and suffix_voc.endswith(
                ar.KASRA):
            encl_vo_no_inflect_mark = enclitic_voc.replace(ar.HEH + ar.DAMMA,
                                                           ar.HEH)
            enclitic_voc = enclitic_voc.replace(ar.HEH + ar.DAMMA,
                                                ar.HEH + ar.KASRA)
            #print "ok"
        return enclitic_voc, encl_vo_no_inflect_mark

    @staticmethod
    def get_word_variant(word, proclitic, suffix, enclitic):
        """
        Get the word variant to be joined to the suffix.
        For example: word = مدرسة, suffix = ي. The word is converted to مدرست.
        @param word: word found in dictionary.
        @type word: unicode.
        @param proclitic: proclitic ( first level).
        @type proclitic: unicode.
        @param suffix: suffix ( first level).
        @type suffix: unicode.
        @param enclitic: enclitic( second level).
        @type enclitic: unicode.
        @return: variant of word.
        @rtype: unicode.
        """
        word_stem = word

        suffix_nm = ar.strip_tashkeel(suffix)

        enclitic_nm = ar.strip_tashkeel(enclitic)
        long_suffix_nm = suffix_nm + enclitic_nm
        #if the word ends by a haraka
        word_stem = ar.strip_lastharaka(word_stem)

        # الاسم المؤنث بالتاء المروبطة نحذفها قبل اللاحقات مثل ات وية
        if word_stem.endswith(ar.TEH_MARBUTA):
            # حالة الاسماء مثل حياة وفتاة
            if word_stem.endswith(ar.ALEF + ar.TEH_MARBUTA):
                if suffix_nm in (ar.YEH, ar.YEH + ar.TEH_MARBUTA,
                                 ar.YEH + ar.ALEF + ar.TEH):
                    word_stem = word_stem[:-1] + ar.TEH
                elif suffix_nm == ar.ALEF + ar.TEH:
                    #نحن بحاجة إلى حذف آخر حركة أيضا
                    word_stem = ar.strip_lastharaka(word_stem[:-1])
                elif long_suffix_nm != u"":
                    word_stem = word_stem[:-1] + ar.TEH

            elif suffix_nm in (ar.ALEF + ar.TEH, ar.YEH + ar.TEH_MARBUTA, ar.YEH,
                               ar.YEH + ar.ALEF + ar.TEH):
                #نحن بحاجة إلى حذف آخر حركة أيضا
                word_stem = ar.strip_lastharaka(word_stem[:-1])
            # الاسم المؤنث بالتاء المروبطة نفتحها قبل اللصق
            #مدرسة +ين = مدرستين
            elif long_suffix_nm != u"":
                word_stem = word_stem[:-1] + ar.TEH

        elif word_stem.endswith(ar.ALEF_MAKSURA):
            # الاسم المقصور إذا اتصل بلاحقة نحوية صارت ألف المقصورة ياء
            # مستوى +ان = مستويان
            # إذا كانت اللاحقة الصرفية ذات حروف تتحول الألف المقصورة إلى ياء
            if suffix_nm != u"":
                word_stem = word_stem[:-1] + ar.YEH
        # إذا كانت اللاحقة الصرفية حركات فقط والضمير المتصل  تتحول الألف المقصورة إلى ألف
            elif enclitic_nm != u"":
                word_stem = word_stem[:-1] + ar.ALEF

        elif word_stem.endswith(ar.KASRA + ar.YEH):
            # الاسم المنقوص ينتهي بياء قبلها مكسور
            # إذا كان لا ضمير واللاحقة فقط حركات
            # نحذف ال
            if u"ال" not in proclitic and u"لل" not in proclitic and not enclitic_nm and not suffix_nm:
                word_stem = ar.strip_lastharaka(word_stem[:-2])
        # الاسم المنقوص ينتهي بياء قبلها مكسور
        # إذا كانت اللاحقة ياء ونون
            elif suffix_nm in (ar.YEH + ar.NOON, ar.WAW + ar.NOON):
                word_stem = ar.strip_lastharaka(word_stem[:-2])

        #ضبط المنتهي بالهمزة حسب حركة اللاحقة النحوية
        elif word_stem.endswith(ar.HAMZA) and (suffix_nm or enclitic_nm):
            if suffix.startswith(ar.DAMMA):
                word_stem = word_stem[:-1] + ar.WAW_HAMZA
            elif suffix.startswith(ar.KASRA):
                word_stem = word_stem[:-1] + ar.YEH_HAMZA
            elif (word_stem.endswith(ar.YEH + ar.HAMZA)
                  or word_stem.endswith(ar.YEH + ar.SUKUN + ar.HAMZA)
                 ) and suffix.startswith(ar.FATHATAN):
                word_stem = word_stem[:-1] + ar.YEH_HAMZA
        return word_stem

    def vocalize(self, noun, proclitic, suffix, enclitic):
        """
        Join the  noun and its affixes, and get the vocalized form
        @param noun: noun found in dictionary.
        @type noun: unicode.
        @param proclitic: first level prefix.
        @type proclitic: unicode.

        @param suffix: second level suffix.
        @type suffix: unicode.
        @param enclitic: first level suffix.
        @type enclitic: unicode.
        @return: vocalized word.
        @rtype: unicode.
        """
        return self.generator.vocalize(noun, proclitic, suffix, enclitic)
        
        
        # proclitic have only an uniq vocalization in arabic
        proclitic_voc = SNC.COMP_PREFIX_LIST_TAGS[proclitic]["vocalized"][0]
        # enclitic can be variant according to suffix
        #print (u"vocalize: '%s' '%s'"%(enclitic, noun)).encode('utf8')
        enclitic_voc = SNC.COMP_SUFFIX_LIST_TAGS[enclitic]["vocalized"][0]
        enclitic_voc, encl_voc_non_inflect = self.get_enclitic_variant(
            enclitic_voc, suffix)

        suffix_voc = suffix
        #adjust some some harakat

        #strip last if tanwin or last harakat
        if ar.is_haraka(noun[-1:]):
            #(DAMMATAN, FATHATAN, KASRATAN, FATHA, DAMMA, KASRA):
            noun = noun[:-1]
        # convert Fathatan into one fatha, in some cases where #
        #the tanwin is not at the end: eg. محتوًى
        noun = noun.replace(ar.FATHATAN, ar.FATHA)

        #add shadda if the first letter is sunny and the proclitic
        #contains AL definition mark
        if u'تعريف' in SNC.COMP_PREFIX_LIST_TAGS[proclitic]["tags"] and ar.is_sun(noun[0]):
            noun = u''.join([noun[0], ar.SHADDA, noun[1:]])
            #strip the Skun from the lam
            if proclitic_voc.endswith(ar.SUKUN):
                proclitic_voc = proclitic_voc[:-1]
        #completate the dictionary word vocalization
        # this allow to avoid some missed harakat before ALEF
        # in the dictionary form of word, all alefat are preceded by Fatha
        #~noun = ar.complet
        #~ print "stem_noun.vocalize; before", noun.encode('utf8');
        noun = noun.replace(ar.ALEF, ar.FATHA + ar.ALEF)
        #~ print "stem_noun.vocalize; 2", noun.encode('utf8');

        noun = noun.replace(ar.ALEF_MAKSURA, ar.FATHA + ar.ALEF_MAKSURA)
        noun = re.sub(u"(%s)+" % ar.FATHA, ar.FATHA, noun)
        # remove initial fatha if alef is the first letter
        noun = re.sub(u"^(%s)+" % ar.FATHA, "", noun)
        #~ print "stem_noun.vocalize; 3", noun.encode('utf8');

        # generate the word variant for some words witch ends by special
        #letters like Teh_marbuta or Alef_maksura, or hamza,
        #the variant is influed by the suffix harakat,
        # for example مدرسة+ي = مدرست+ي
        mankous = True if noun.endswith(ar.KASRA + ar.YEH) else False

        noun = self.get_word_variant(noun, proclitic, suffix, enclitic)

        # generate the suffix variant. if the suffix is Teh_marbuta or
        #Alef_maksura, or hamza, the variant is influed by the enclitic harakat,
        # for example مدرس+ة+ي = مدرس+ت+ي
        suffix_voc, suffix_non_irab_mark = self.get_suffix_variants(
            noun, suffix_voc, enclitic, mankous)

        # generate the non vacalized end word: the vocalized word
        # without the I3rab Mark
        # if the suffix is a short haraka
        word_non_irab_mark = ''.join(
            [proclitic_voc, noun, suffix_non_irab_mark, encl_voc_non_inflect])

        #generate vocalized form

        word_vocalized = ''.join([proclitic_voc, noun, suffix_voc, enclitic_voc])
        #used for spelling purposes
        segmented = '-'.join([proclitic_voc, noun, suffix_voc, enclitic_voc])
        segmented = ar.strip_tashkeel(segmented)
        #~word_vocalized = ar.ajust_vocalization(word_vocalized)3
        for patrn, repl in SNC.AJUST_VOCAL_PATTERNS:
            word_vocalized = word_vocalized.replace(patrn, repl)
            word_non_irab_mark = word_non_irab_mark.replace(patrn, repl)

        return word_vocalized, word_non_irab_mark, segmented
    @staticmethod
    def get_noun_variants(noun):
        """ generate noun varaintes """
        # if the word contains ALEF8MADDA, we convert it into 2 HAMZA above ALEF
        noun_list = []
        if ar.ALEF_MADDA in noun:
            noun_list.append(
                noun.replace(ar.ALEF_MADDA, ar.ALEF_HAMZA_ABOVE * 2))
            noun_list.append(
                noun.replace(ar.ALEF_MADDA, ar.HAMZA +ar.ALEF))
            noun_list.append(
                noun.replace(ar.ALEF_MADDA, ar.ALEF_HAMZA_ABOVE +ar.ALEF))
        return noun_list

    @staticmethod
    def get_input_stem_variants(stem, enclitic_nm):
        """ generate stem varaintes """
        list_stem = []
        if enclitic_nm:
            # حالة الاسم المقصور إذا كان به زيادة مثل سوى +ها = سواها
            if stem.endswith(ar.ALEF):
                list_stem.append(stem[:-1] + ar.ALEF_MAKSURA)
            # حالة المؤنث بالتاء قد يكون أصلها تاء مربوطة
            elif stem.endswith(ar.TEH):
                list_stem.append(stem[:-1] + ar.TEH_MARBUTA)
        # Case of Mankous Name حالة الاسم المنقوص
        # إذا لم يكن به زيادة ربما كان الاسم منقوصا منوّنا
        # قد يقبل السابقة مثل وقاضٍ
        # لكنه لا يقبل اللاحقة
        else:  # no enclitic
            list_stem.append(stem + ar.YEH)
        return list_stem

    @staticmethod
    def verify_affix(word, list_seg, affix_list):
        """
        Verify possible affixes in the resulted segments according
        to the given affixes list.
        @param word: the input word.
        @type word: unicode.
        @param list_seg: list of word segments indexes (numbers).
        @type list_seg: list of pairs.
        @return: list of acceped segments.
        @rtype: list of pairs.
        """
        return [
            s for s in list_seg
            if '-'.join([word[:s[0]], word[s[1]:]]) in affix_list
        ]

    @staticmethod
    def validate_tags(noun_tuple, affix_tags, proclitic_nm, enclitic_nm,
                      suffix_nm):
        """
        Test if the given word from dictionary is compabilbe with affixes tags.
        @param noun_tuple: the input word attributes given from dictionary.
        @type noun_tuple: dict.
        @param affix_tags: a list of tags given by affixes.
        @type affix_tags:list.
        @param proclitic_nm: first level prefix vocalized.
        @type proclitic_nm: unicode.
        @param enclitic_nm: first level suffix vocalized.
        @type enclitic_nm: unicode.
        @param suffix_nm: first level suffix vocalized.
        @type suffix_nm: unicode.
        @return: if the tags are compatible.
        @rtype: Boolean.
        """
        #~ proclitic = ar.strip_tashkeel(proclitic)
        proclitic = proclitic_nm
        #~ enclitic = enclitic_nm
        #~ suffix = suffix_nm
        if u'تنوين' in affix_tags and noun_tuple['mamnou3_sarf']:
            return False
        # ألجمع السالم لا يتصل بجمع التكسير
        if noun_tuple['number'] in (u'جمع', u'جمع تكسير'):

            if u'جمع مؤنث سالم' in affix_tags:
                return False
            if u'جمع مذكر سالم' in affix_tags:
                return False
            if u'مثنى' in affix_tags:
                return False
        #تدقيق الغضافة إلى الضمائر المتصلة
        if enclitic_nm in (u"هم", u"هن", u"كما", u"كم",
                           u"هما") and not noun_tuple['hm_suffix']:
            return False
        if enclitic_nm in (u"ه", u"ها") and not noun_tuple['ha_suffix']:
            return False
        #حالة قابلية السوابق  بدون تعريف
        if u"ال" not in proclitic and not noun_tuple['k_prefix']:
            return False
        #حالة قابلية السوابق  مع التعريف
        if proclitic.endswith(u"ال") and not noun_tuple['kal_prefix']:
            return False
        # حالة المضاف إلى ما بعهده في حالة جمع المذكر السالم
        # مثل لاعبو، رياضيو
        if suffix_nm == ar.WAW and not noun_tuple['w_suffix']:
            return False
        #التاء المربوطة لا تتصل بجمع التكسير
        if suffix_nm == ar.TEH_MARBUTA and noun_tuple['number'] == u"جمع":
            return False
        # elif  u'مضاف' in affix_tags and not noun_tuple['annex']:
        # return False


    #todo
    # u'mankous':8,
    # u'feminable':9, *
    # u'number':10,
    # u'dualable':11, *
    # u'masculin_plural':12, *
    # u'feminin_plural':13, *
    # u'broken_plural':14,
    # u'mamnou3_sarf':15,
    # u'relative':16,
    # u'w_suffix':17,
    # u'hm_suffix':18, *
    # u'kal_prefix':19, *
    # u'ha_suffix':20, *
    # u'k_suffix':21, *
    # u'annex':22,
        return True


def mainly():
    """
    Test main"""
    #ToDo: use the full dictionary of arramooz
    wordlist = [
        u'يضرب',
        u'الكتاب',
        u'الاستخدام',
        u"فاستعمالهم",
        u'ضرب',
        u'لأكلهم',
    ]
    nounstemmer = NounStemmer()
    nounstemmer.set_debug(True)
    for word in wordlist:
        nounstemmer.conj_stemmer.segment(word)
        print(nounstemmer.conj_stemmer.get_affix_list())
    for word in wordlist:
        result = nounstemmer.stemming_noun(word)
        for analyzed in result:
            #~ print(repr(analyzed).encode('utf8')) #.decode('unicode-escape')
            #~ print(u'\n'.join(analyzed.__dict__.keys()))
            for key in analyzed.__dict__.keys():
                print((u'\t'.join([
                    key,
                    repr(analyzed.__dict__[key]).decode('unicode-escape')
                ]))).encode('utf8')
            print()
            print()


#Class test
if __name__ == '__main__':
    mainly()



#!/usr/bin/python
# -*- coding=utf-8 -*-
#-------------------------------------------------------------------------------
# Name:        CArabic Dictionary from Arramooz Al Waseet
# Purpose:     Morphological porpus Dictionary.
#
# Author:      Taha Zerrouki (taha.zerrouki[at]gmail.com)
#
# Created:     16-12-2013
# Copyright:   (c) Taha Zerrouki 2013
# Licence:     GPL
#-------------------------------------------------------------------------------
"""
Arabic Dictionary Class from Arramooz Al Waseet.
Used in multiporpus morpholigical treatment
"""
from __future__ import (
    absolute_import,
    print_function,
    unicode_literals,
    division,
    )
import re
import os, os.path
#from pysqlite2 import dbapi2 as sqlite
import sqlite3 as sqlite
import sys
FILE_DB = u"data/custom_dictionary.sqlite"
import pyarabic.araby as araby
import arramooz.arabicdictionary
class custom_dictionary(arramooz.arabicdictionary.ArabicDictionary):
    """
    Arabic dictionary Class customized, used to add new words before add them to arramooz project.
    It's used to add new entries manually to this dictionary by user, 
    In order to improve lately Arramooz project.
    """

    def __init__(self, table_name, filename_db=FILE_DB):
        """
        initialisation of dictionary from a data dictionary, create indexes 
        to speed up the access.

        """
        
        arramooz.arabicdictionary.ArabicDictionary.__init__(self, table_name)

        # costumized data
        # get the database path
        if hasattr(sys, 'frozen'): # only when running in py2exe this exists
            base = sys.prefix
        else: # otherwise this is a regular python script
            base = os.path.dirname(os.path.realpath(__file__))
        file_path = os.path.join(base, filename_db)

        if os.path.exists(file_path):
            try:
                self.db_connect = sqlite.connect(file_path)                
                self.db_connect.row_factory = sqlite.Row 
                self.cursor = self.db_connect.cursor()
                #print("Connect with success")
            except  IOError:
                print("Fatal Error Can't find the database file", file_path)
        else:
            print(u" ".join(["Inexistant File", file_path, " current dir ", 
            os.curdir]).encode('utf8'))

#Class test
def mainly():
    """
    main test
    """
    #ToDo: use the full dictionary of arramooz
    mydict = custom_dictionary('verbs')
    wordlist = [u"استقلّ", u'استقل', u"كذب"]
    tmp_list = []
    for word in wordlist:
        foundlist = mydict.lookup(word)
        for word_tuple in foundlist:
            word_tuple = dict(word_tuple)
            vocalized = word_tuple['vocalized']
            tmp_list.append(dict(word_tuple))
    print(repr(tmp_list).replace('},','},\n').decode("unicode-escape"))            
if __name__  ==  '__main__':
    mainly()



#!/usr/bin/python
# -*- coding=utf-8 -*-
#-------------------------------------------------------------------------------
# Name:        analex
# Purpose:     Arabic lexical analyser, provides feature to stem arabic 
#words as noun, verb, stopword
#
# Author:      Taha Zerrouki (taha.zerrouki[at]gmail.com)
#
# Created:     31-10-2011
# Copyright:   (c) Taha Zerrouki 2011
# Licence:     GPL
#-------------------------------------------------------------------------------
"""Cache Module for analex"""
import sys

    
from hashlib import md5
import os
from codernitydb3.database import Database
from codernitydb3.hash_index import HashIndex
    
#~ class WithAIndex(HashIndex):

    #~ def __init__(self, *args, **kwargs):
        #~ kwargs['key_format'] = '32s'
        #~ super(WithAIndex, self).__init__(*args, **kwargs)

    #~ def make_key_value(self, data):
        #~ a_val = data.get("a")
        #~ if a_val is not None:
            #~ return a_val, None
        #~ return None

    #~ def make_key(self, key):
        #~ return key
class WithAIndex(HashIndex):
    def __init__(self, *args, **kwargs):
        # kwargs['entry_line_format'] = '<32s32sIIcI'
        #~ kwargs['key_format'] = '16s'
        kwargs['key_format'] = '32s'
        kwargs['hash_lim'] = 8 * 1024
        super(WithAIndex, self).__init__(*args, **kwargs)

    def make_key_value(self, data):
        a_val = data.get("a")
        if a_val:
            if not isinstance(a_val, str):
                a_val = str(a_val)
            return md5(a_val.encode('utf8')).digest(), None
        return None

    def make_key(self, key):
        if not isinstance(key, str):
            key = str(key)
        return md5(key.encode('utf8')).digest()





class Cache(object):
    """
        cache for word morphological analysis
    """
    def __init__(self, dp_path = False):
        """
        Create Analex Cache
        """
        DB_PATH = os.path.join(os.path.expanduser('~'), '.qalsadiCache')
        self.cache = {
            'checkedWords': {},
            'FreqWords': {
                'noun': {},
                'verb': {},
                'stopword': {}
            },
        }
        if not dp_path:
            dp_path = self.DB_PATH
        else:
            dp_path = os.path.join(os.path.dirname(dp_path), '.qalsadiCache')
        #~ sys.stderr.write("Tahahahah\n"+" "+ dp_path +" "+str(type(dp_path)))
        #~ self.db = Database("/tmp/QC")
        try:
        #~ if True:
            self.db = Database(str(dp_path))
            if not self.db.exists():
                self.db.create()
                x_ind = WithAIndex(self.db.path, "a")
                self.db.add_index(x_ind)
            else:
                self.db.open()
        except:
        #~ else:
            print("Can't Open data base")
            self.db = None
    def __del__(self):
        """
        Delete instance and clear cache

        """
        self.cache = None
        if self.db:
            self.db.close()

    def is_already_checked(self, word):
        """ return if ``word`` is already cached"""
        try:
            return bool(self.db.get('a', word))
        except:
            return False
        #~ except: return False;

    def get_checked(self, word):
        """ return checked ``word`` form cache"""
        #~ word = bytes(word, "utf-8")
        if self.db:
            xxx = self.db.get('a', word, with_doc=True)
            yyy = xxx.get('doc', False)
            if yyy:
                return yyy.get('d', [])
            else: return []
        else: return []

    def add_checked(self, word, data):
        """ add checked ``word`` form cache"""
        #~ word = bytes(word, "utf-8")        
        idata = {"a": word, 'd': data}
        if self.db:
            self.db.insert(idata)

    def exists_cache_freq(self, word, wordtype):
        """ return if word exists in freq cache"""
        return word in self.cache['FreqWords']

    def get_freq(self, originalword, wordtype):
        """ return  ``word`` frequency form cache"""
        return self.cache['FreqWords'][wordtype].get(originalword, 0)

    def add_freq(self, original, wordtype, freq):
        """ add   ``original`` frequency ``freq`` to cache"""
        self.cache['FreqWords'][wordtype][original] = freq


def mainly():
    """main function"""
    print("test")
    path = os.path.join(os.path.dirname(__file__),"cache", '.qalsadiCache')
    cacher = Cache(path)
    word = "taha"
    data = "zerrouki"
    cacher.add_checked(word, data)
    d2  = cacher.get_checked(word)
    print(d2)

if __name__ == "__main__":
    mainly()
